# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, unicode_literals

import io
import logging
import os
import re

import salt.utils.files
import salt.utils.platform
import salt.utils.win_reg as reg
from tests.support.case import ModuleCase
from tests.support.helpers import destructiveTest, random_string
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import skipIf

log = logging.getLogger(__name__)


@skipIf(not salt.utils.platform.is_windows(), "windows test only")
class WinLgpoTest(ModuleCase):
    """
    Tests for salt.modules.win_lgpo
    """

    osrelease = None

    def _testRegistryPolicy(
        self,
        policy_name,
        policy_config,
        registry_value_hive,
        registry_value_path,
        registry_value_vname,
        expected_value_data,
    ):
        """
        Takes a registry based policy name and config and validates that the
        expected registry value exists and has the correct data

        policy_name
            name of the registry based policy to configure
        policy_config
            the configuration of the policy
        registry_value_hive
            the registry hive that the policy registry path is in
        registry_value_path
            the registry value path that the policy updates
        registry_value_vname
            the registry value name
        expected_value_data
            the expected data that the value will contain
        """
        ret = self.run_function(
            "lgpo.set_computer_policy", (policy_name, policy_config)
        )
        self.assertTrue(ret)
        val = reg.read_value(
            hive=registry_value_hive,
            key=registry_value_path,
            vname=registry_value_vname,
        )
        self.assertTrue(
            val["success"],
            msg="Failed to obtain the registry data for policy {0}".format(policy_name),
        )
        if val["success"]:
            self.assertEqual(
                val["vdata"],
                expected_value_data,
                "The registry value data {0} does not match the expected value {1} for policy {2}".format(
                    val["vdata"], expected_value_data, policy_name
                ),
            )

    def _testSeceditPolicy(
        self,
        policy_name,
        policy_config,
        expected_regexes,
        cumulative_rights_assignments=True,
    ):
        """
        Takes a secedit policy name and config and validates that the expected
        output is returned from secedit

        policy_name
            name of the secedit policy to configure
        policy_config
            the configuration of the policy
        expected_regexes
            the expected regexes to be found in the secedit output file
        """
        ret = self.run_function(
            "lgpo.set_computer_policy",
            (policy_name, policy_config),
            cumulative_rights_assignments=cumulative_rights_assignments,
        )
        self.assertTrue(ret)
        secedit_output_file = os.path.join(
            RUNTIME_VARS.TMP, random_string("secedit-output-")
        )
        secedit_output = self.run_function(
            "cmd.run", (), cmd="secedit /export /cfg {0}".format(secedit_output_file)
        )
        secedit_file_content = None
        if secedit_output:
            with io.open(secedit_output_file, encoding="utf-16") as _reader:
                secedit_file_content = _reader.read()
        for expected_regex in expected_regexes:
            match = re.search(
                expected_regex, secedit_file_content, re.IGNORECASE | re.MULTILINE
            )
            self.assertIsNotNone(
                match,
                'Failed validating policy "{0}" configuration, regex "{1}" not found in secedit output'.format(
                    policy_name, expected_regex
                ),
            )

    def _testAdmxPolicy(
        self,
        policy_name,
        policy_config,
        expected_regexes,
        assert_true=True,
        policy_class="Machine",
    ):
        """
        Takes a ADMX policy name and config and validates that the expected
        output is returned from lgpo looking at the Registry.pol file

        policy_name
            name of the ADMX policy to configure
        policy_config
            the configuration of the policy
        expected_regexes
            the expected regexes to be found in the lgpo parse output
        assert_true
            set to false if expecting the module run to fail
        policy_class
            the policy class this policy belongs to, either Machine or User
        """
        lgpo_function = "set_computer_policy"
        lgpo_class = "/m"
        lgpo_folder = "Machine"
        if policy_class.lower() == "user":
            lgpo_function = "set_user_policy"
            lgpo_class = "/u"
            lgpo_folder = "User"

        ret = self.run_function(
            "lgpo.{0}".format(lgpo_function), (policy_name, policy_config)
        )
        log.debug("lgpo set_computer_policy ret == %s", ret)
        cmd = [
            "lgpo.exe",
            "/parse",
            lgpo_class,
            r"c:\Windows\System32\GroupPolicy\{}\Registry.pol".format(lgpo_folder),
        ]
        if assert_true:
            self.assertTrue(ret)
            lgpo_output = self.run_function("cmd.run", (), cmd=" ".join(cmd))
            # validate that the lgpo output doesn't say the format is invalid
            self.assertIsNone(
                re.search(r"Invalid file format\.", lgpo_output, re.IGNORECASE),
                msg="Failed validating Registry.pol file format",
            )
            # validate that the regexes we expect are in the output
            for expected_regex in expected_regexes:
                match = re.search(expected_regex, lgpo_output, re.IGNORECASE)
                self.assertIsNotNone(
                    match,
                    msg='Failed validating policy "{0}" configuration, regex '
                    '"{1}" not found in lgpo output:\n{2}'
                    "".format(policy_name, expected_regex, lgpo_output),
                )
        else:
            # expecting it to fail
            self.assertNotEqual(ret, True)

    def runTest(self):
        """
        runTest method
        """

    @classmethod
    def setUpClass(cls):
        """
        class setup function, only runs once

        downloads and extracts the lgpo.exe tool into c:/windows/system32
        for use in validating the registry.pol files

        gets osrelease grain for tests that are only applicable to certain
        windows versions
        """
        osrelease_grains = cls().run_function("grains.item", ["osrelease"])
        if "osrelease" in osrelease_grains:
            cls.osrelease = osrelease_grains["osrelease"]
        else:
            log.debug("Unable to get osrelease grain")
        if not os.path.exists(r"c:\windows\system32\lgpo.exe"):
            log.debug("lgpo.exe does not exist, attempting to download/extract")
            ret = cls().run_function(
                "state.single",
                ("archive.extracted", r"c:\windows\system32"),
                source="https://download.microsoft.com/download/8/5/C/85C25433-A1B0-4FFA-9429-7E023E7DA8D8/LGPO.zip",
                archive_format="zip",
                source_hash="sha256=6ffb6416366652993c992280e29faea3507b5b5aa661c33ba1af31f48acea9c4",
                enforce_toplevel=False,
            )
            log.debug("ret from archive.unzip == %s", ret)

    @destructiveTest
    def test_set_user_policy_point_and_print_restrictions(self):
        """
        Test setting/unsetting/changing the PointAndPrint_Restrictions user policy
        """
        # Disable Point and Print Restrictions
        self._testAdmxPolicy(
            r"Control Panel\Printers\Point and Print Restrictions",
            "Disabled",
            [
                r"User[\s]*Software\\Policies\\Microsoft\\Windows NT\\Printers\\PointAndPrint[\s]*Restricted[\s]*DWORD:0",
                r"User[\s]*Software\\Policies\\Microsoft\\Windows NT\\Printers\\PointAndPrint[\s]*TrustedServers[\s]*DELETE",
                r"User[\s]*Software\\Policies\\Microsoft\\Windows NT\\Printers\\PointAndPrint[\s]*ServerList[\s]*DELETE",
                r"User[\s]*Software\\Policies\\Microsoft\\Windows NT\\Printers\\PointAndPrint[\s]*InForest[\s]*DELETE",
                r"User[\s]*Software\\Policies\\Microsoft\\Windows NT\\Printers\\PointAndPrint[\s]*NoWarningNoElevationOnInstall[\s]*DELETE",
                r"User[\s]*Software\\Policies\\Microsoft\\Windows NT\\Printers\\PointAndPrint[\s]*UpdatePromptSettings[\s]*DELETE",
            ],
            policy_class="User",
        )
        # Enable Point and Print Restrictions
        self._testAdmxPolicy(
            r"Point and Print Restrictions",
            {
                "Users can only point and print to these servers": True,
                "Enter fully qualified server names separated by semicolons": "fakeserver1;fakeserver2",
                "Users can only point and print to machines in their forest": True,
                "When installing drivers for a new connection": "Show warning and elevation prompt",
                "When updating drivers for an existing connection": "Show warning only",
            },
            [
                r"User[\s]*Software\\Policies\\Microsoft\\Windows NT\\Printers\\PointAndPrint[\s]*Restricted[\s]*DWORD:1",
                r"User[\s]*Software\\Policies\\Microsoft\\Windows NT\\Printers\\PointAndPrint[\s]*TrustedServers[\s]*DWORD:1",
                r"User[\s]*Software\\Policies\\Microsoft\\Windows NT\\Printers\\PointAndPrint[\s]*ServerList[\s]*SZ:fakeserver1;fakeserver2",
                r"User[\s]*Software\\Policies\\Microsoft\\Windows NT\\Printers\\PointAndPrint[\s]*InForest[\s]*DWORD:1",
                r"User[\s]*Software\\Policies\\Microsoft\\Windows NT\\Printers\\PointAndPrint[\s]*NoWarningNoElevationOnInstall[\s]*DWORD:0",
                r"User[\s]*Software\\Policies\\Microsoft\\Windows NT\\Printers\\PointAndPrint[\s]*UpdatePromptSettings[\s]*DWORD:1",
            ],
            policy_class="User",
        )
        # set Point and Print Restrictions to 'Not Configured'
        self._testAdmxPolicy(
            r"Control Panel\Printers\Point and Print Restrictions",
            "Not Configured",
            [
                r"; Source file:  c:\\windows\\system32\\grouppolicy\\user\\registry.pol[\s]*; PARSING COMPLETED."
            ],
            policy_class="User",
        )

    @destructiveTest
    def test_set_computer_policy_NTP_Client(self):
        """
        Test setting/unsetting/changing NTP Client policies
        """
        # Disable Configure NTP Client
        self._testAdmxPolicy(
            r"System\Windows Time Service\Time Providers\Configure Windows NTP Client",
            "Disabled",
            [
                r"Computer[\s]*Software\\Policies\\Microsoft\\W32time\\Parameters[\s]*NtpServer[\s]*DELETE",
                r"Computer[\s]*Software\\Policies\\Microsoft\\W32time\\Parameters[\s]*Type[\s]*DELETE",
                r"Computer[\s]*Software\\Policies\\Microsoft\\W32time\\TimeProviders\\NtpClient[\s]*CrossSiteSyncFlags[\s]*DELETE",
                r"Computer[\s]*Software\\Policies\\Microsoft\\W32time\\TimeProviders\\NtpClient[\s]*ResolvePeerBackoffMinutes[\s]*DELETE",
                r"Computer[\s]*Software\\Policies\\Microsoft\\W32time\\TimeProviders\\NtpClient[\s]*ResolvePeerBackoffMaxTimes[\s]*DELETE",
                r"Computer[\s]*Software\\Policies\\Microsoft\\W32time\\TimeProviders\\NtpClient[\s]*SpecialPollInterval[\s]*DELETE",
                r"Computer[\s]*Software\\Policies\\Microsoft\\W32time\\TimeProviders\\NtpClient[\s]*EventLogFlags[\s]*DELETE",
            ],
        )
        # Enable Configure NTP Client
        self._testAdmxPolicy(
            r"System\Windows Time Service\Time Providers\Configure Windows NTP Client",
            {
                "NtpServer": "time.windows.com,0x9",
                "Type": "NT5DS",
                "CrossSiteSyncFlags": 2,
                "ResolvePeerBackoffMinutes": 15,
                "ResolvePeerBackoffMaxTimes": 7,
                "W32TIME_SpecialPollInterval": 3600,
                "W32TIME_NtpClientEventLogFlags": 0,
            },
            [
                r"Computer[\s]*Software\\Policies\\Microsoft\\W32time\\Parameters[\s]*NtpServer[\s]*SZ:time.windows.com,0x9",
                r"Computer[\s]*Software\\Policies\\Microsoft\\W32time\\Parameters[\s]*Type[\s]*SZ:NT5DS",
                r"Computer[\s]*Software\\Policies\\Microsoft\\W32time\\TimeProviders\\NtpClient[\s]*CrossSiteSyncFlags[\s]*DWORD:2",
                r"Computer[\s]*Software\\Policies\\Microsoft\\W32time\\TimeProviders\\NtpClient[\s]*ResolvePeerBackoffMinutes[\s]*DWORD:15",
                r"Computer[\s]*Software\\Policies\\Microsoft\\W32time\\TimeProviders\\NtpClient[\s]*ResolvePeerBackoffMaxTimes[\s]*DWORD:7",
                r"Computer[\s]*Software\\Policies\\Microsoft\\W32time\\TimeProviders\\NtpClient[\s]*SpecialPollInterval[\s]*DWORD:3600",
                r"Computer[\s]*Software\\Policies\\Microsoft\\W32time\\TimeProviders\\NtpClient[\s]*EventLogFlags[\s]*DWORD:0",
            ],
        )
        # set Configure NTP Client to 'Not Configured'
        self._testAdmxPolicy(
            r"System\Windows Time Service\Time Providers\Configure Windows NTP Client",
            "Not Configured",
            [
                r"; Source file:  c:\\windows\\system32\\grouppolicy\\machine\\registry.pol[\s]*; PARSING COMPLETED."
            ],
        )

    @destructiveTest
    def test_set_computer_policy_RA_Unsolicit(self):
        """
        Test setting/unsetting/changing RA_Unsolicit policy
        """

        # Disable RA_Unsolicit
        log.debug("Attempting to disable RA_Unsolicit")
        self._testAdmxPolicy(
            "RA_Unsolicit",
            "Disabled",
            [
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows NT\\Terminal Services[\s]*fAllowUnsolicited[\s]*DWORD:0",
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows NT\\Terminal Services[\s]*fAllowUnsolicitedFullControl[\s]*DELETE",
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows NT\\Terminal Services\\RAUnsolicit[\s]*\*[\s]*DELETEALLVALUES",
            ],
        )
        # configure RA_Unsolicit
        log.debug("Attempting to configure RA_Unsolicit")
        self._testAdmxPolicy(
            "RA_Unsolicit",
            {
                "Configure Offer Remote Access": "Enabled",
                "Permit remote control of this computer": "Allow helpers to remotely control the computer",
                "Helpers": ["administrators", "user1"],
            },
            [
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows NT\\Terminal Services\\RAUnsolicit[\s]*user1[\s]*SZ:user1[\s]*",
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows NT\\Terminal Services\\RAUnsolicit[\s]*administrators[\s]*SZ:administrators[\s]*",
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows NT\\Terminal Services[\s]*fAllowUnsolicited[\s]*DWORD:1",
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows NT\\Terminal Services[\s]*fAllowUnsolicitedFullControl[\s]*DWORD:1",
            ],
        )
        # Not Configure RA_Unsolicit
        log.debug("Attempting to set RA_Unsolicit to Not Configured")
        self._testAdmxPolicy(
            "RA_Unsolicit",
            "Not Configured",
            [
                r"; Source file:  c:\\windows\\system32\\grouppolicy\\machine\\registry.pol[\s]*; PARSING COMPLETED."
            ],
        )

    @destructiveTest
    def test_set_computer_policy_Pol_HardenedPaths(self):
        # Disable Pol_HardenedPaths
        log.debug("Attempting to disable Pol_HardenedPaths")
        self._testAdmxPolicy(
            "Pol_HardenedPaths",
            "Disabled",
            [
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows\\NetworkProvider\\HardenedPaths[\s]*\*[\s]*DELETEALLVALUES"
            ],
        )
        # Configure Pol_HardenedPaths
        log.debug("Attempting to configure Pol_HardenedPaths")
        self._testAdmxPolicy(
            "Pol_HardenedPaths",
            {
                "Hardened UNC Paths": {
                    r"\\*\NETLOGON": "RequireMutualAuthentication=1, RequireIntegrity=1",
                    r"\\*\SYSVOL": "RequireMutualAuthentication=1, RequireIntegrity=1",
                }
            },
            [
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows\\NetworkProvider\\HardenedPaths[\s]*\\\\\*\\NETLOGON[\s]*SZ:RequireMutualAuthentication=1, RequireIntegrity=1[\s]*",
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows\\NetworkProvider\\HardenedPaths[\s]*\\\\\*\\SYSVOL[\s]*SZ:RequireMutualAuthentication=1, RequireIntegrity=1[\s]*",
            ],
        )
        # Not Configure Pol_HardenedPaths
        log.debug("Attempting to set Pol_HardenedPaths to Not Configured")
        self._testAdmxPolicy(
            "Pol_HardenedPaths",
            "Not Configured",
            [
                r"; Source file:  c:\\windows\\system32\\grouppolicy\\machine\\registry.pol[\s]*; PARSING COMPLETED."
            ],
        )

    @destructiveTest
    def test_set_computer_policy_WindowsUpdate(self):
        """
        Test setting/unsetting/changing WindowsUpdate policy
        """
        # Configure Automatic Updates has different options in different builds
        # and releases of Windows, so we'll get the elements and add them if
        # they are present. Newer elements will need to be added manually as
        # they are released by Microsoft
        result = self.run_function(
            "lgpo.get_policy_info",
            ["Configure Automatic Updates"],
            policy_class="machine",
        )
        the_policy = {}
        the_policy_check_enabled = [
            r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*NoAutoUpdate[\s]*DWORD:0",
        ]
        the_policy_check_disabled = [
            r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*NoAutoUpdate[\s]*DWORD:1",
        ]
        for item in result["policy_elements"]:
            if "Configure automatic updating" in item["element_aliases"]:
                the_policy.update(
                    {
                        "Configure automatic updating": "4 - Auto download and schedule the install",
                    }
                )
                the_policy_check_enabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*AUOptions[\s]*DWORD:4",
                )
                the_policy_check_disabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*AUOptions[\s]*DELETE",
                )
            elif "Install during automatic maintenance" in item["element_aliases"]:
                the_policy.update({"Install during automatic maintenance": True})
                the_policy_check_enabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*AutomaticMaintenanceEnabled[\s]*DWORD:1\s*",
                )
                the_policy_check_disabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*AutomaticMaintenanceEnabled[\s]*DELETE",
                )
            elif "Scheduled install day" in item["element_aliases"]:
                the_policy.update({"Scheduled install day": "7 - Every Saturday"})
                the_policy_check_enabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*ScheduledInstallDay[\s]*DWORD:7",
                )
                the_policy_check_disabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*ScheduledInstallDay[\s]*DELETE",
                )
            elif "Scheduled install time" in item["element_aliases"]:
                the_policy.update({"Scheduled install time": "17:00"})
                the_policy_check_enabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*ScheduledInstallTime[\s]*DWORD:17",
                )
                the_policy_check_disabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*ScheduledInstallTime[\s]*DELETE",
                )
            elif (
                "Install updates for other Microsoft products"
                in item["element_aliases"]
            ):
                the_policy.update(
                    {"Install updates for other Microsoft products": True}
                )
                the_policy_check_enabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*AllowMUUpdateService[\s]*DWORD:1\s*"
                )
                the_policy_check_disabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*AllowMUUpdateService[\s]*DELETE"
                )
            elif "AutoUpdateSchEveryWeek" in item["element_aliases"]:
                the_policy.update({"AutoUpdateSchEveryWeek": True})
                the_policy_check_enabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*ScheduledInstallEveryWeek[\s]*DWORD:1\s*"
                )
                the_policy_check_disabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*ScheduledInstallEveryWeek[\s]*DELETE"
                )
            elif "First week of the month" in item["element_aliases"]:
                the_policy.update({"First week of the month": True})
                the_policy_check_enabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*ScheduledInstallFirstWeek[\s]*DWORD:1\s*"
                )
                the_policy_check_disabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*ScheduledInstallFirstWeek[\s]*DELETE"
                )
            elif "Second week of the month" in item["element_aliases"]:
                the_policy.update({"Second week of the month": True})
                the_policy_check_enabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*ScheduledInstallSecondWeek[\s]*DWORD:1\s*"
                )
                the_policy_check_disabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*ScheduledInstallSecondWeek[\s]*DELETE"
                )
            elif "Third week of the month" in item["element_aliases"]:
                the_policy.update({"Third week of the month": True})
                the_policy_check_enabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*ScheduledInstallThirdWeek[\s]*DWORD:1\s*"
                )
                the_policy_check_disabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*ScheduledInstallThirdWeek[\s]*DELETE"
                )
            elif "Fourth week of the month" in item["element_aliases"]:
                the_policy.update({"Fourth week of the month": True})
                the_policy_check_enabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*ScheduledInstallFourthWeek[\s]*DWORD:1\s*"
                )
                the_policy_check_disabled.append(
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*ScheduledInstallFourthWeek[\s]*DELETE"
                )

        # enable Automatic Updates
        self._testAdmxPolicy(
            r"Windows Components\Windows Update\Configure Automatic Updates",
            the_policy,
            the_policy_check_enabled,
        )

        # disable Configure Automatic Updates
        self._testAdmxPolicy(
            r"Windows Components\Windows Update\Configure Automatic Updates",
            "Disabled",
            the_policy_check_disabled,
        )

        # set Configure Automatic Updates to 'Not Configured'
        self._testAdmxPolicy(
            r"Windows Components\Windows Update\Configure Automatic Updates",
            "Not Configured",
            [
                r"; Source file:  c:\\windows\\system32\\grouppolicy\\machine\\registry.pol[\s]*; PARSING COMPLETED."
            ],
        )

    @destructiveTest
    def test_set_computer_policy_ClipboardRedirection(self):
        """
        Test setting/unsetting/changing ClipboardRedirection policy
        """
        # Enable/Disable/Not Configured "Do not allow Clipboard redirection"
        self._testAdmxPolicy(
            r"Windows Components\Remote Desktop Services\Remote Desktop Session Host\Device and Resource Redirection\Do not allow Clipboard redirection",
            "Enabled",
            [
                r"Computer[\s]*Software\\Policies\\Microsoft\\Windows NT\\Terminal Services[\s]*fDisableClip[\s]*DWORD:1"
            ],
        )
        self._testAdmxPolicy(
            r"Windows Components\Remote Desktop Services\Remote Desktop Session Host\Device and Resource Redirection\Do not allow Clipboard redirection",
            "Disabled",
            [
                r"Computer[\s]*Software\\Policies\\Microsoft\\Windows NT\\Terminal Services[\s]*fDisableClip[\s]*DWORD:0"
            ],
        )
        self._testAdmxPolicy(
            r"Windows Components\Remote Desktop Services\Remote Desktop Session Host\Device and Resource Redirection\Do not allow Clipboard redirection",
            "Not Configured",
            [
                r"; Source file:  c:\\windows\\system32\\grouppolicy\\machine\\registry.pol[\s]*; PARSING COMPLETED."
            ],
        )

    @destructiveTest
    def test_set_computer_policy_LockoutDuration(self):
        """
        Test setting LockoutDuration
        """
        # For LockoutDuration to be meaningful, first configure
        # LockoutThreshold
        self._testSeceditPolicy("LockoutThreshold", 3, [r"^LockoutBadCount = 3"])

        # Next set the LockoutDuration non-zero value, as this is required
        # before setting LockoutWindow
        self._testSeceditPolicy("LockoutDuration", 60, [r"^LockoutDuration = 60"])

        # Now set LockoutWindow to a valid value <= LockoutDuration. If this
        # is not set, then the LockoutDuration zero value is ignored by the
        # Windows API (leading to a false sense of accomplishment)
        self._testSeceditPolicy("LockoutWindow", 60, [r"^ResetLockoutCount = 60"])

        # set LockoutDuration zero value, the secedit zero value is -1
        self._testSeceditPolicy("LockoutDuration", 0, [r"^LockoutDuration = -1"])

    @destructiveTest
    def test_set_computer_policy_GuestAccountStatus(self):
        """
        Test setting/unsetting/changing GuestAccountStatus
        """
        # disable GuestAccountStatus
        self._testSeceditPolicy(
            "GuestAccountStatus", "Disabled", [r"^EnableGuestAccount = 0"]
        )
        # enable GuestAccountStatus
        self._testSeceditPolicy(
            "GuestAccountStatus", "Enabled", [r"^EnableGuestAccount = 1"]
        )

    @destructiveTest
    def test_set_computer_policy_PasswordComplexity(self):
        """
        Test setting/unsetting/changing PasswordComplexity
        """
        # disable PasswordComplexity
        self._testSeceditPolicy(
            "Password must meet complexity requirements",
            "Disabled",
            [r"^PasswordComplexity = 0"],
        )
        # enable PasswordComplexity
        self._testSeceditPolicy(
            "PasswordComplexity", "Enabled", [r"^PasswordComplexity = 1"]
        )

    @destructiveTest
    def test_set_computer_policy_PasswordLen(self):
        """
        Test setting/unsetting/changing PasswordLength
        """
        # set Minimum password length
        self._testSeceditPolicy(
            "Minimum password length", 10, [r"^MinimumPasswordLength = 10"]
        )
        # set MinimumPasswordLength = 0
        self._testSeceditPolicy("MinPasswordLen", 0, [r"^MinimumPasswordLength = 0"])

    @destructiveTest
    def test_set_computer_policy_SeNetworkLogonRight(self):
        """
        Test setting/unsetting/changing PasswordLength
        """
        # set SeNetworkLogonRight to only Administrators
        self._testSeceditPolicy(
            "Access this computer from the network",
            ["Administrators"],
            [r"^SeNetworkLogonRight = \*S-1-5-32-544"],
            cumulative_rights_assignments=False,
        )
        # set SeNetworkLogonRight back to the default
        self._testSeceditPolicy(
            "SeNetworkLogonRight",
            ["Everyone", "Administrators", "Users", "Backup Operators"],
            [
                r"^SeNetworkLogonRight = \*S-1-1-0,\*S-1-5-32-544,\*S-1-5-32-545,\*S-1-5-32-551"
            ],
        )

    @destructiveTest
    def test_set_computer_policy_multipleAdmxPolicies(self):
        """
        Tests setting several ADMX policies in succession and validating the configuration w/lgop
        """
        # set one policy
        self._testAdmxPolicy(
            r"Windows Components\Remote Desktop Services\Remote Desktop Session Host\Device and Resource Redirection\Do not allow Clipboard redirection",
            "Disabled",
            [
                r"Computer[\s]*Software\\Policies\\Microsoft\\Windows NT\\Terminal Services[\s]*fDisableClip[\s]*DWORD:0"
            ],
        )

        # set another policy and make sure both this policy and the previous are okay
        self._testAdmxPolicy(
            "RA_Unsolicit",
            {
                "Configure Offer Remote Access": "Enabled",
                "Permit remote control of this computer": "Allow helpers to remotely control the computer",
                "Helpers": ["administrators", "user1"],
            },
            [
                r"Computer[\s]*Software\\Policies\\Microsoft\\Windows NT\\Terminal Services[\s]*fDisableClip[\s]*DWORD:0",
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows NT\\Terminal Services\\RAUnsolicit[\s]*user1[\s]*SZ:user1[\s]*",
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows NT\\Terminal Services\\RAUnsolicit[\s]*administrators[\s]*SZ:administrators[\s]*",
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows NT\\Terminal Services[\s]*fAllowUnsolicited[\s]*DWORD:1",
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows NT\\Terminal Services[\s]*fAllowUnsolicitedFullControl[\s]*DWORD:1",
            ],
        )
        # Configure Automatic Updates and validate everything is still okay
        self._testAdmxPolicy(
            r"Windows Components\Windows Update\Configure Automatic Updates",
            "Disabled",
            [
                r"Computer[\s]*Software\\Policies\\Microsoft\\Windows NT\\Terminal Services[\s]*fDisableClip[\s]*DWORD:0",
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows NT\\Terminal Services\\RAUnsolicit[\s]*user1[\s]*SZ:user1[\s]*",
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows NT\\Terminal Services\\RAUnsolicit[\s]*administrators[\s]*SZ:administrators[\s]*",
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows NT\\Terminal Services[\s]*fAllowUnsolicited[\s]*DWORD:1",
                r"Computer[\s]*Software\\policies\\Microsoft\\Windows NT\\Terminal Services[\s]*fAllowUnsolicitedFullControl[\s]*DWORD:1",
                r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*NoAutoUpdate[\s]*DWORD:1",
                r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*AUOptions[\s]*DELETE",
                r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*AutomaticMaintenanceEnabled[\s]*DELETE",
                r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*ScheduledInstallDay[\s]*DELETE",
                r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*ScheduledInstallTime[\s]*DELETE",
                r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate\\AU[\s]*AllowMUUpdateService[\s]*DELETE",
            ],
        )

    @destructiveTest
    def test_set_computer_policy_DisableDomainCreds(self):
        """
        Tests Enable/Disable of DisableDomainCreds policy
        """
        self._testRegistryPolicy(
            policy_name="DisableDomainCreds",
            policy_config="Enabled",
            registry_value_hive="HKEY_LOCAL_MACHINE",
            registry_value_path="SYSTEM\\CurrentControlSet\\Control\\Lsa",
            registry_value_vname="DisableDomainCreds",
            expected_value_data=1,
        )
        self._testRegistryPolicy(
            policy_name="Network access: Do not allow storage of passwords and credentials for network authentication",
            policy_config="Disabled",
            registry_value_hive="HKEY_LOCAL_MACHINE",
            registry_value_path="SYSTEM\\CurrentControlSet\\Control\\Lsa",
            registry_value_vname="DisableDomainCreds",
            expected_value_data=0,
        )

    @destructiveTest
    def test_set_computer_policy_ForceGuest(self):
        """
        Tests changing ForceGuest policy
        """
        self._testRegistryPolicy(
            policy_name="ForceGuest",
            policy_config="Guest only - local users authenticate as Guest",
            registry_value_hive="HKEY_LOCAL_MACHINE",
            registry_value_path="SYSTEM\\CurrentControlSet\\Control\\Lsa",
            registry_value_vname="ForceGuest",
            expected_value_data=1,
        )
        self._testRegistryPolicy(
            policy_name="Network access: Sharing and security model for local accounts",
            policy_config="Classic - local users authenticate as themselves",
            registry_value_hive="HKEY_LOCAL_MACHINE",
            registry_value_path="SYSTEM\\CurrentControlSet\\Control\\Lsa",
            registry_value_vname="ForceGuest",
            expected_value_data=0,
        )

    @destructiveTest
    def test_set_computer_policy_DisableUXWUAccess(self):
        """
        Tests changing DisableUXWUAccess
        #50079 shows using the 'Remove access to use all Windows Update features' failed
        Policy only exists on 2016
        """
        valid_osreleases = ["2016Server"]
        if self.osrelease not in valid_osreleases:
            self.skipTest(
                "DisableUXWUAccess policy is only applicable if the osrelease grain is {0}".format(
                    " or ".join(valid_osreleases)
                )
            )
        else:
            self._testAdmxPolicy(
                r"DisableUXWUAccess",
                "Enabled",
                [
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate[\s]*SetDisableUXWUAccess[\s]*DWORD:1"
                ],
            )
            self._testAdmxPolicy(
                r"Remove access to use all Windows Update features",
                "Disabled",
                [
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate[\s]*SetDisableUXWUAccess[\s]*DWORD:0"
                ],
            )
            self._testAdmxPolicy(
                r"Windows Components\Windows Update\Remove access to use all Windows Update features",
                "Not Configured",
                [
                    r"; Source file:  c:\\windows\\system32\\grouppolicy\\machine\\registry.pol[\s]*; PARSING COMPLETED."
                ],
            )

    @destructiveTest
    def test_set_computer_policy_Access_data_sources_across_domains(self):
        """
        Tests that a policy that has multiple names
        """
        self._testAdmxPolicy(
            r"Access data sources across domains", "Enabled", [], assert_true=False
        )
        self._testAdmxPolicy(
            r"Windows Components\Internet Explorer\Internet Control Panel\Security Page\Internet Zone\Access data sources across domains",
            {"Access data sources across domains": "Prompt"},
            [
                r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\CurrentVersion\\Internet Settings\\Zones\\3[\s]*1406[\s]*DWORD:1"
            ],
        )
        self._testAdmxPolicy(
            r"Windows Components\Internet Explorer\Internet Control Panel\Security Page\Internet Zone\Access data sources across domains",
            {"Access data sources across domains": "Enable"},
            [
                r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\CurrentVersion\\Internet Settings\\Zones\\3[\s]*1406[\s]*DWORD:0"
            ],
        )
        self._testAdmxPolicy(
            r"Windows Components\Internet Explorer\Internet Control Panel\Security Page\Internet Zone\Access data sources across domains",
            "Disabled",
            [
                r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\CurrentVersion\\Internet Settings\\Zones\\3[\s]*1406[\s]*DELETE"
            ],
        )

    @destructiveTest
    def test_set_computer_policy_ActiveHours(self):
        """
        Test configuring the ActiveHours policy, #47784
        Only applies to 2016Server
        # activehours.sls
            active_hours_policy:
              lgpo.set:
                - computer_policy:
                    'ActiveHours':
                        'ActiveHoursStartTime': '8 AM'
                        'ActiveHoursEndTime': '7 PM'
        """
        valid_osreleases = ["2016Server"]
        if self.osrelease not in valid_osreleases:
            self.skipTest(
                "ActiveHours policy is only applicable if the osrelease grain is {0}".format(
                    " or ".join(valid_osreleases)
                )
            )
        else:
            self._testAdmxPolicy(
                r"ActiveHours",
                {"ActiveHoursStartTime": "8 AM", "ActiveHoursEndTime": "7 PM"},
                [
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate[\s]*SetActiveHours[\s]*DWORD:1",
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate[\s]*ActiveHoursStart[\s]*DWORD:8",
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate[\s]*ActiveHoursEnd[\s]*DWORD:19",
                ],
            )
            self._testAdmxPolicy(
                r"ActiveHours",
                {"ActiveHoursStartTime": "5 AM", "ActiveHoursEndTime": "10 PM"},
                [
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate[\s]*SetActiveHours[\s]*DWORD:1",
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate[\s]*ActiveHoursStart[\s]*DWORD:5",
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate[\s]*ActiveHoursEnd[\s]*DWORD:22",
                ],
            )
            self._testAdmxPolicy(
                "Turn off auto-restart for updates during active hours",
                "Disabled",
                [
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate[\s]*SetActiveHours[\s]*DWORD:0",
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate[\s]*ActiveHoursStart[\s]*DELETE",
                    r"Computer[\s]*Software\\Policies\\Microsoft\\Windows\\WindowsUpdate[\s]*ActiveHoursEnd[\s]*DELETE",
                ],
            )
            self._testAdmxPolicy(
                r"Windows Components\Windows Update\Turn off auto-restart for updates during active hours",
                "Not Configured",
                [
                    r"; Source file:  c:\\windows\\system32\\grouppolicy\\machine\\registry.pol[\s]*; PARSING COMPLETED."
                ],
            )

    @destructiveTest
    def test_set_computer_policy_AllowTelemetry(self):
        """
        Tests that a the AllowTelemetry policy is applied correctly and that it
        doesn't appear in subsequent group policy states as having changed
        """
        valid_osreleases = ["10", "2016Server", "2019Server"]
        if self.osrelease not in valid_osreleases:
            self.skipTest(
                "Allow Telemetry policy is only applicable if the "
                "osrelease grain is {0}".format(" or ".join(valid_osreleases))
            )
        else:
            self._testAdmxPolicy(
                "Allow Telemetry",
                {"AllowTelemetry": "1 - Basic"},
                [
                    r"Software\\Policies\\Microsoft\\Windows\\DataCollection[\s]*AllowTelemetry[\s]*DWORD:1"
                ],
                assert_true=True,
            )
            # This policy does not exist on newer Windows builds
            result = self.run_function(
                "lgpo.get_policy_info",
                ["Disable pre-release features or settings"],
                policy_class="machine",
            )
            if result["policy_found"]:
                result = self.run_function(
                    "state.single",
                    ["lgpo.set"],
                    name="state",
                    computer_policy={
                        "Disable pre-release features or settings": "Disabled"
                    },
                )
                name = "lgpo_|-state_|-state_|-set"
                expected = {
                    "new": {
                        "Computer Configuration": {
                            "Disable pre-release features or settings": "Disabled"
                        }
                    },
                    "old": {
                        "Computer Configuration": {
                            "Disable pre-release features or settings": "Not Configured"
                        }
                    },
                }
                self.assertDictEqual(result[name]["changes"], expected)
            else:
                result = self.run_function(
                    "lgpo.get_policy_info",
                    ["Manage preview builds"],
                    policy_class="machine",
                )
                if result["policy_found"]:
                    result = self.run_function(
                        "state.single",
                        ["lgpo.set"],
                        name="state",
                        computer_policy={"Manage preview builds": "Disabled"},
                    )
                    name = "lgpo_|-state_|-state_|-set"
                    expected = {
                        "new": {
                            "Computer Configuration": {
                                "Manage preview builds": "Disabled"
                            }
                        },
                        "old": {
                            "Computer Configuration": {
                                "Manage preview builds": "Not Configured"
                            }
                        },
                    }
                    self.assertDictEqual(result[name]["changes"], expected)

    def tearDown(self):
        """
        tearDown method, runs after each test
        """
        self.run_state(
            "file.absent",
            name="c:\\windows\\system32\\grouppolicy\\machine\\registry.pol",
        )
        self.run_state(
            "file.absent", name="c:\\windows\\system32\\grouppolicy\\user\\registry.pol"
        )



# -*- coding: utf-8 -*-
"""
    tests.integration.conftest
    ~~~~~~~~~~~~~~~~~~~~~~~~~~

    Integration tests PyTest configuration/fixtures
"""
# pylint: disable=unused-argument,redefined-outer-name

from __future__ import absolute_import, unicode_literals

import logging

import pytest
from tests.support.pytest.fixtures import *  # pylint: disable=unused-wildcard-import

log = logging.getLogger(__name__)


# @pytest.fixture(scope='package')
# def salt_syndic_master(request, salt_factories):
#     return salt_factories.spawn_master(request, 'syndic_master', order_masters=True)


# @pytest.fixture(scope='package')
# def salt_syndic(request, salt_factories, salt_syndic_master):
#     return salt_factories.spawn_syndic(request, 'syndic', master_of_masters_id='syndic_master')


# @pytest.fixture(scope='package')
# def salt_master(request, salt_factories, salt_syndic_master):
#    return salt_factories.spawn_master(request, 'master', master_of_masters_id='syndic_master')


@pytest.fixture(scope="package")
def salt_master(request, salt_factories):
    return salt_factories.spawn_master(request, "master")


@pytest.fixture(scope="package")
def salt_minion(request, salt_factories, salt_master):
    proc = salt_factories.spawn_minion(request, "minion", master_id="master")
    # Sync All
    salt_call_cli = salt_factories.get_salt_call_cli("minion")
    ret = salt_call_cli.run("saltutil.sync_all", _timeout=120)
    assert ret.exitcode == 0, ret
    return proc


@pytest.fixture(scope="package")
def salt_sub_minion(request, salt_factories, salt_master):
    proc = salt_factories.spawn_minion(request, "sub_minion", master_id="master")
    # Sync All
    salt_call_cli = salt_factories.get_salt_call_cli("sub_minion")
    ret = salt_call_cli.run("saltutil.sync_all", _timeout=120)
    assert ret.exitcode == 0, ret
    return proc


@pytest.fixture(scope="package", autouse=True)
def bridge_pytest_and_runtests_integration(
    bridge_pytest_and_runtests,
    salt_factories,
    # salt_syndic_master,
    # salt_syndic,
    salt_master,
    salt_minion,
):

    yield


@pytest.fixture(scope="package")
def salt_cli(salt_factories, salt_minion, salt_master):
    return salt_factories.get_salt_cli(salt_master.config["id"])


@pytest.fixture(scope="package")
def salt_cp_cli(salt_factories, salt_minion, salt_master):
    return salt_factories.get_salt_cp_cli(salt_master.config["id"])


@pytest.fixture(scope="package")
def salt_key_cli(salt_factories, salt_minion, salt_master):
    return salt_factories.get_salt_key_cli(salt_master.config["id"])


@pytest.fixture(scope="package")
def salt_run_cli(salt_factories, salt_minion, salt_master):
    return salt_factories.get_salt_run_cli(salt_master.config["id"])


@pytest.fixture(scope="package")
def salt_call_cli(salt_factories, salt_minion, salt_master):
    return salt_factories.get_salt_call_cli(salt_minion.config["id"])



# -*- coding: utf-8 -*-
"""
Test the verification routines
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import ctypes
import getpass
import os
import shutil
import socket
import stat
import sys
import tempfile

# Import salt libs
import salt.utils.files
import salt.utils.platform

# Import 3rd-party libs
from salt.ext import six
from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
from salt.utils.verify import (
    check_max_open_files,
    check_user,
    clean_path,
    log,
    valid_id,
    verify_env,
    verify_log,
    verify_log_files,
    verify_logs_filter,
    verify_socket,
    zmq_version,
)
from tests.support.helpers import TstSuiteLoggingHandler, requires_network
from tests.support.mock import MagicMock, patch

# Import Salt Testing libs
# Import Salt Testing libs
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase, skipIf

# Import third party libs
if sys.platform.startswith("win"):
    import win32file
else:
    import resource


# Import third party libs
if sys.platform.startswith("win"):
    import win32file
else:
    import resource


class TestVerify(TestCase):
    """
    Verify module tests
    """

    def test_valid_id_exception_handler(self):
        """
        Ensure we just return False if we pass in invalid or undefined paths.
        Refs #8259
        """
        opts = {"pki_dir": "/tmp/whatever"}
        self.assertFalse(valid_id(opts, None))

    def test_valid_id_pathsep(self):
        """
        Path separators in id should make it invalid
        """
        opts = {"pki_dir": "/tmp/whatever"}
        # We have to test both path separators because os.path.normpath will
        # convert forward slashes to backslashes on Windows.
        for pathsep in ("/", "\\"):
            self.assertFalse(valid_id(opts, pathsep.join(("..", "foobar"))))

    def test_zmq_verify(self):
        self.assertTrue(zmq_version())

    def test_zmq_verify_insufficient(self):
        import zmq

        with patch.object(zmq, "__version__", "2.1.0"):
            self.assertFalse(zmq_version())

    def test_user(self):
        self.assertTrue(check_user(getpass.getuser()))

    def test_no_user(self):
        # Catch sys.stderr here since no logging is configured and
        # check_user WILL write to sys.stderr
        class FakeWriter(object):
            def __init__(self):
                self.output = ""

            def write(self, data):
                self.output += data

        stderr = sys.stderr
        writer = FakeWriter()
        sys.stderr = writer
        # Now run the test
        if sys.platform.startswith("win"):
            self.assertTrue(check_user("nouser"))
        else:
            self.assertFalse(check_user("nouser"))
        # Restore sys.stderr
        sys.stderr = stderr
        if writer.output != 'CRITICAL: User not found: "nouser"\n':
            # If there's a different error catch, write it to sys.stderr
            sys.stderr.write(writer.output)

    @skipIf(salt.utils.platform.is_windows(), "No verify_env Windows")
    def test_verify_env(self):
        root_dir = tempfile.mkdtemp(dir=RUNTIME_VARS.TMP)
        var_dir = os.path.join(root_dir, "var", "log", "salt")
        key_dir = os.path.join(root_dir, "key_dir")
        verify_env([var_dir], getpass.getuser(), root_dir=root_dir)
        self.assertTrue(os.path.exists(var_dir))
        dir_stat = os.stat(var_dir)
        self.assertEqual(dir_stat.st_uid, os.getuid())
        self.assertEqual(dir_stat.st_mode & stat.S_IRWXU, stat.S_IRWXU)
        self.assertEqual(dir_stat.st_mode & stat.S_IRWXG, 40)
        self.assertEqual(dir_stat.st_mode & stat.S_IRWXO, 5)

    @requires_network(only_local_network=True)
    def test_verify_socket(self):
        self.assertTrue(verify_socket("", 18000, 18001))
        if socket.has_ipv6:
            # Only run if Python is built with IPv6 support; otherwise
            # this will just fail.
            try:
                self.assertTrue(verify_socket("::", 18000, 18001))
            except socket.error as serr:
                # Python has IPv6 enabled, but the system cannot create
                # IPv6 sockets (otherwise the test would return a bool)
                # - skip the test
                #
                # FIXME - possibly emit a message that the system does
                # not support IPv6.
                pass

    def test_max_open_files(self):
        with TstSuiteLoggingHandler() as handler:
            logmsg_dbg = "DEBUG:This salt-master instance has accepted {0} minion keys."
            logmsg_chk = (
                "{0}:The number of accepted minion keys({1}) should be lower "
                "than 1/4 of the max open files soft setting({2}). According "
                "to the system's hard limit, there's still a margin of {3} "
                "to raise the salt's max_open_files setting. Please consider "
                "raising this value."
            )
            logmsg_crash = (
                "{0}:The number of accepted minion keys({1}) should be lower "
                "than 1/4 of the max open files soft setting({2}). "
                "salt-master will crash pretty soon! According to the "
                "system's hard limit, there's still a margin of {3} to "
                "raise the salt's max_open_files setting. Please consider "
                "raising this value."
            )
            if sys.platform.startswith("win"):
                logmsg_crash = (
                    "{0}:The number of accepted minion keys({1}) should be lower "
                    "than 1/4 of the max open files soft setting({2}). "
                    "salt-master will crash pretty soon! Please consider "
                    "raising this value."
                )

            if sys.platform.startswith("win"):
                # Check the Windows API for more detail on this
                # http://msdn.microsoft.com/en-us/library/xt874334(v=vs.71).aspx
                # and the python binding http://timgolden.me.uk/pywin32-docs/win32file.html
                mof_s = mof_h = win32file._getmaxstdio()
            else:
                mof_s, mof_h = resource.getrlimit(resource.RLIMIT_NOFILE)
            tempdir = tempfile.mkdtemp(prefix="fake-keys")
            keys_dir = os.path.join(tempdir, "minions")
            os.makedirs(keys_dir)

            mof_test = 256

            if sys.platform.startswith("win"):
                win32file._setmaxstdio(mof_test)
            else:
                resource.setrlimit(resource.RLIMIT_NOFILE, (mof_test, mof_h))

            try:
                prev = 0
                for newmax, level in (
                    (24, None),
                    (66, "INFO"),
                    (127, "WARNING"),
                    (196, "CRITICAL"),
                ):

                    for n in range(prev, newmax):
                        kpath = os.path.join(keys_dir, six.text_type(n))
                        with salt.utils.files.fopen(kpath, "w") as fp_:
                            fp_.write(
                                str(n)
                            )  # future lint: disable=blacklisted-function

                    opts = {"max_open_files": newmax, "pki_dir": tempdir}

                    check_max_open_files(opts)

                    if level is None:
                        # No log message is triggered, only the DEBUG one which
                        # tells us how many minion keys were accepted.
                        self.assertEqual([logmsg_dbg.format(newmax)], handler.messages)
                    else:
                        self.assertIn(logmsg_dbg.format(newmax), handler.messages)
                        self.assertIn(
                            logmsg_chk.format(
                                level,
                                newmax,
                                mof_test,
                                mof_test - newmax
                                if sys.platform.startswith("win")
                                else mof_h - newmax,
                            ),
                            handler.messages,
                        )
                    handler.clear()
                    prev = newmax

                newmax = mof_test
                for n in range(prev, newmax):
                    kpath = os.path.join(keys_dir, six.text_type(n))
                    with salt.utils.files.fopen(kpath, "w") as fp_:
                        fp_.write(str(n))  # future lint: disable=blacklisted-function

                opts = {"max_open_files": newmax, "pki_dir": tempdir}

                check_max_open_files(opts)
                self.assertIn(logmsg_dbg.format(newmax), handler.messages)
                self.assertIn(
                    logmsg_crash.format(
                        "CRITICAL",
                        newmax,
                        mof_test,
                        mof_test - newmax
                        if sys.platform.startswith("win")
                        else mof_h - newmax,
                    ),
                    handler.messages,
                )
                handler.clear()
            except IOError as err:
                if err.errno == 24:
                    # Too many open files
                    self.skipTest("We've hit the max open files setting")
                raise
            finally:
                if sys.platform.startswith("win"):
                    win32file._setmaxstdio(mof_h)
                else:
                    resource.setrlimit(resource.RLIMIT_NOFILE, (mof_s, mof_h))
                shutil.rmtree(tempdir)

    def test_verify_log(self):
        """
        Test that verify_log works as expected
        """
        message = (
            "Insecure logging configuration detected! Sensitive data may be logged."
        )

        mock_cheese = MagicMock()
        with patch.object(log, "warning", mock_cheese):
            verify_log({"log_level": "cheeseshop"})
            mock_cheese.assert_called_once_with(message)

        mock_trace = MagicMock()
        with patch.object(log, "warning", mock_trace):
            verify_log({"log_level": "trace"})
            mock_trace.assert_called_once_with(message)

        mock_none = MagicMock()
        with patch.object(log, "warning", mock_none):
            verify_log({})
            mock_none.assert_called_once_with(message)

        mock_info = MagicMock()
        with patch.object(log, "warning", mock_info):
            verify_log({"log_level": "info"})
            self.assertTrue(mock_info.call_count == 0)


class TestVerifyLog(TestCase):
    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    def test_verify_logs_filter(self):
        filtered = verify_logs_filter(
            ["udp://foo", "tcp://bar", "/tmp/foo", "file://tmp/bar"]
        )
        assert filtered == ["/tmp/foo"], filtered

    @skipIf(salt.utils.platform.is_windows(), "Not applicable on Windows")
    def test_verify_log_files_udp_scheme(self):
        verify_log_files(["udp://foo"], getpass.getuser())
        self.assertFalse(os.path.isdir(os.path.join(os.getcwd(), "udp:")))

    @skipIf(salt.utils.platform.is_windows(), "Not applicable on Windows")
    def test_verify_log_files_tcp_scheme(self):
        verify_log_files(["udp://foo"], getpass.getuser())
        self.assertFalse(os.path.isdir(os.path.join(os.getcwd(), "tcp:")))

    @skipIf(salt.utils.platform.is_windows(), "Not applicable on Windows")
    def test_verify_log_files_file_scheme(self):
        verify_log_files(["file://{}"], getpass.getuser())
        self.assertFalse(os.path.isdir(os.path.join(os.getcwd(), "file:")))

    @skipIf(salt.utils.platform.is_windows(), "Not applicable on Windows")
    def test_verify_log_files(self):
        path = os.path.join(self.tmpdir, "foo", "bar.log")
        self.assertFalse(os.path.exists(path))
        verify_log_files([path], getpass.getuser())
        self.assertTrue(os.path.exists(path))


class TestCleanPath(TestCase):
    """
    salt.utils.clean_path works as expected
    """

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    def test_clean_path_valid(self):
        path_a = os.path.join(self.tmpdir, "foo")
        path_b = os.path.join(self.tmpdir, "foo", "bar")
        assert clean_path(path_a, path_b) == path_b

    def test_clean_path_invalid(self):
        path_a = os.path.join(self.tmpdir, "foo")
        path_b = os.path.join(self.tmpdir, "baz", "bar")
        assert clean_path(path_a, path_b) == ""


__CSL = None


def symlink(source, link_name):
    """
    symlink(source, link_name) Creates a symbolic link pointing to source named
    link_name
    """
    global __CSL
    if __CSL is None:
        csl = ctypes.windll.kernel32.CreateSymbolicLinkW
        csl.argtypes = (ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_uint32)
        csl.restype = ctypes.c_ubyte
        __CSL = csl
    flags = 0
    if source is not None and os.path.isdir(source):
        flags = 1
    if __CSL(link_name, source, flags) == 0:
        raise ctypes.WinError()


@skipIf(six.PY2 and salt.utils.platform.is_windows(), "Skipped on windows py2")
class TestCleanPathLink(TestCase):
    """
    Ensure salt.utils.clean_path works with symlinked directories and files
    """

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()
        self.to_path = os.path.join(self.tmpdir, "linkto")
        self.from_path = os.path.join(self.tmpdir, "linkfrom")
        if six.PY2 or salt.utils.platform.is_windows():
            kwargs = {}
        else:
            kwargs = {"target_is_directory": True}
        if salt.utils.platform.is_windows():
            symlink(self.to_path, self.from_path, **kwargs)
        else:
            os.symlink(self.to_path, self.from_path, **kwargs)

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    def test_clean_path_symlinked_src(self):
        test_path = os.path.join(self.from_path, "test")
        expect_path = os.path.join(self.to_path, "test")
        ret = clean_path(self.from_path, test_path)
        assert ret == expect_path, "{} is not {}".format(ret, expect_path)

    def test_clean_path_symlinked_tgt(self):
        test_path = os.path.join(self.to_path, "test")
        expect_path = os.path.join(self.to_path, "test")
        ret = clean_path(self.from_path, test_path)
        assert ret == expect_path, "{} is not {}".format(ret, expect_path)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Alexander Pyatkin <asp@thexyz.net>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.webutil as htpasswd

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class HtpasswdTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.webutil
    """

    def setup_loader_modules(self):
        return {htpasswd: {"__opts__": {"test": False}}}

    def test_user_exists_already(self):
        """
        Test if it returns True when user already exists in htpasswd file
        """

        mock = MagicMock(return_value={"retcode": 0})

        with patch.dict(htpasswd.__salt__, {"file.grep": mock}):
            ret = htpasswd.user_exists("larry", "badpass", "/etc/httpd/htpasswd")
            expected = {
                "name": "larry",
                "result": True,
                "comment": "User already known",
                "changes": {},
            }
            self.assertEqual(ret, expected)

    def test_new_user_success(self):
        """
        Test if it returns True when new user is added to htpasswd file
        """

        mock_grep = MagicMock(return_value={"retcode": 1})
        mock_useradd = MagicMock(return_value={"retcode": 0, "stderr": "Success"})

        with patch.dict(
            htpasswd.__salt__, {"file.grep": mock_grep, "webutil.useradd": mock_useradd}
        ):
            ret = htpasswd.user_exists("larry", "badpass", "/etc/httpd/htpasswd")
            expected = {
                "name": "larry",
                "result": True,
                "comment": "Success",
                "changes": {"larry": True},
            }
            self.assertEqual(ret, expected)

    def test_new_user_error(self):
        """
        Test if it returns False when adding user to htpasswd failed
        """

        mock_grep = MagicMock(return_value={"retcode": 1})
        mock_useradd = MagicMock(return_value={"retcode": 1, "stderr": "Error"})

        with patch.dict(
            htpasswd.__salt__, {"file.grep": mock_grep, "webutil.useradd": mock_useradd}
        ):
            ret = htpasswd.user_exists("larry", "badpass", "/etc/httpd/htpasswd")
            expected = {
                "name": "larry",
                "result": False,
                "comment": "Error",
                "changes": {},
            }
            self.assertEqual(ret, expected)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

import os
import time

# Import Salt libs
import salt.utils.decorators
from tests.support.runtests import RUNTIME_VARS

EXIT_CODE_SH = os.path.join(RUNTIME_VARS.BASE_FILES, "exit_code.sh")
EXIT_CODE_CMD = os.path.join(RUNTIME_VARS.BASE_FILES, "exit_code.cmd")


def _exit_code(code):
    if os.name == "nt":
        return "cmd /c {0} {1}".format(EXIT_CODE_CMD, code)
    else:
        return "/usr/bin/env sh {0} {1}".format(EXIT_CODE_SH, code)


def _fallbackfunc():
    return False, "fallback"


def working_function():
    return True


@salt.utils.decorators.depends(True)
def booldependsTrue():
    return True


@salt.utils.decorators.depends(False)
def booldependsFalse():
    return True


@salt.utils.decorators.depends("time")
def depends():
    ret = {"ret": True, "time": time.time()}
    return ret


@salt.utils.decorators.depends("time123")
def missing_depends():
    return True


@salt.utils.decorators.depends("time", fallback_function=_fallbackfunc)
def depends_will_not_fallback():
    ret = {"ret": True, "time": time.time()}
    return ret


@salt.utils.decorators.depends("time123", fallback_function=_fallbackfunc)
def missing_depends_will_fallback():
    ret = {"ret": True, "time": time.time()}
    return ret


@salt.utils.decorators.depends(_exit_code(42), retcode=42)
def command_success_retcode():
    return True


@salt.utils.decorators.depends(_exit_code(42), retcode=0)
def command_failure_retcode():
    return True


@salt.utils.decorators.depends(_exit_code(42), nonzero_retcode=True)
def command_success_nonzero_retcode_true():
    return True


@salt.utils.decorators.depends(_exit_code(0), nonzero_retcode=True)
def command_failure_nonzero_retcode_true():
    return True


@salt.utils.decorators.depends(_exit_code(0), nonzero_retcode=False)
def command_success_nonzero_retcode_false():
    return True


@salt.utils.decorators.depends(_exit_code(42), nonzero_retcode=False)
def command_failure_nonzero_retcode_false():
    return True


# The 'depends_versioned.py'-module has __version__ = '1.8'
@salt.utils.decorators.depends("depends_versioned", version="1.0")
def version_depends_false():
    return True


@salt.utils.decorators.depends("depends_versioned", version="2.0")
def version_depends_true():
    return True


# The 'depends_versionless.py'-module does not have a `__version__`-string
@salt.utils.decorators.depends("depends_versionless", version="0.2")
def version_depends_versionless_true():
    return True



# -*- coding: utf-8 -*-
"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.http as http

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class HttpTestCase(TestCase, LoaderModuleMockMixin):
    """
        Validate the HTTP state
    """

    def setup_loader_modules(self):
        return {http: {}}

    def test_query(self):
        """
            Test to perform an HTTP query and statefully return the result
        """
        ret = [
            {
                "changes": {},
                "comment": " Either match text (match) or a "
                "status code (status) is required.",
                "data": {},
                "name": "salt",
                "result": False,
            },
            {
                "changes": {},
                "comment": " (TEST MODE)",
                "data": True,
                "name": "salt",
                "result": None,
            },
        ]
        self.assertDictEqual(http.query("salt"), ret[0])

        with patch.dict(http.__opts__, {"test": True}):
            mock = MagicMock(return_value=True)
            with patch.dict(http.__salt__, {"http.query": mock}):
                self.assertDictEqual(http.query("salt", "Dude", "stack"), ret[1])

    def test_query_pcre_statustype(self):
        """
            Test to perform an HTTP query with a regex used to match the status code and statefully return the result
        """
        testurl = "salturl"
        http_result = {"text": "This page returned a 201 status code", "status": "201"}
        state_return = {
            "changes": {},
            "comment": 'Match text "This page returned" was found. Status pattern "200|201" was found.',
            "data": {"status": "201", "text": "This page returned a 201 status code"},
            "name": testurl,
            "result": True,
        }

        with patch.dict(http.__opts__, {"test": False}):
            mock = MagicMock(return_value=http_result)
            with patch.dict(http.__salt__, {"http.query": mock}):
                self.assertDictEqual(
                    http.query(
                        testurl,
                        match="This page returned",
                        status="200|201",
                        status_type="pcre",
                    ),
                    state_return,
                )

    def test_query_stringstatustype(self):
        """
            Test to perform an HTTP query with a string status code and statefully return the result
        """
        testurl = "salturl"
        http_result = {"text": "This page returned a 201 status code", "status": "201"}
        state_return = {
            "changes": {},
            "comment": 'Match text "This page returned" was found. Status 201 was found.',
            "data": {"status": "201", "text": "This page returned a 201 status code"},
            "name": testurl,
            "result": True,
        }

        with patch.dict(http.__opts__, {"test": False}):
            mock = MagicMock(return_value=http_result)
            with patch.dict(http.__salt__, {"http.query": mock}):
                self.assertDictEqual(
                    http.query(
                        testurl,
                        match="This page returned",
                        status="201",
                        status_type="string",
                    ),
                    state_return,
                )

    def test_query_liststatustype(self):
        """
            Test to perform an HTTP query with a list of status codes and statefully return the result
        """
        testurl = "salturl"
        http_result = {"text": "This page returned a 201 status code", "status": "201"}
        state_return = {
            "changes": {},
            "comment": 'Match text "This page returned" was found. Status 201 was found.',
            "data": {"status": "201", "text": "This page returned a 201 status code"},
            "name": testurl,
            "result": True,
        }

        with patch.dict(http.__opts__, {"test": False}):
            mock = MagicMock(return_value=http_result)
            with patch.dict(http.__salt__, {"http.query": mock}):
                self.assertDictEqual(
                    http.query(
                        testurl,
                        match="This page returned",
                        status=["200", "201"],
                        status_type="list",
                    ),
                    state_return,
                )

    def test_wait_for_with_interval(self):
        """
        Test for wait_for_successful_query waits for request_interval
        """

        query_mock = MagicMock(side_effect=[{"error": "error"}, {"result": True}])

        with patch.object(http, "query", query_mock):
            with patch("time.sleep", MagicMock()) as sleep_mock:
                self.assertEqual(
                    http.wait_for_successful_query(
                        "url", request_interval=1, status=200
                    ),
                    {"result": True},
                )
                sleep_mock.assert_called_once_with(1)

    def test_wait_for_without_interval(self):
        """
        Test for wait_for_successful_query waits for request_interval
        """

        query_mock = MagicMock(side_effect=[{"error": "error"}, {"result": True}])

        with patch.object(http, "query", query_mock):
            with patch("time.sleep", MagicMock()) as sleep_mock:
                self.assertEqual(
                    http.wait_for_successful_query("url", status=200), {"result": True}
                )
                sleep_mock.assert_not_called()



# -*- coding: utf-8 -*-
"""
    :codeauthor: Gareth J. Greenaway <gareth@saltstack.com>
"""

from __future__ import absolute_import

import copy
import logging

import salt.ext.tornado
import salt.ext.tornado.testing
import salt.minion
import salt.syspaths
from tests.support.helpers import slowTest
from tests.support.mixins import AdaptedConfigurationTestCaseMixin
from tests.support.unit import TestCase

log = logging.getLogger(__name__)
__opts__ = {}


class ProxyMinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
    @slowTest
    def test_post_master_init_metaproxy_called(self):
        """
        Tests that when the _post_master_ini function is called, _metaproxy_call is also called.
        """
        mock_opts = salt.config.DEFAULT_MINION_OPTS.copy()
        mock_jid_queue = [123]
        proxy_minion = salt.minion.ProxyMinion(
            mock_opts,
            jid_queue=copy.copy(mock_jid_queue),
            io_loop=salt.ext.tornado.ioloop.IOLoop(),
        )
        try:
            ret = proxy_minion._post_master_init("dummy_master")
            self.assert_called_once(salt.minion._metaproxy_call)
        finally:
            proxy_minion.destroy()

    @slowTest
    def test_handle_decoded_payload_metaproxy_called(self):
        """
        Tests that when the _handle_decoded_payload function is called, _metaproxy_call is also called.
        """
        mock_opts = salt.config.DEFAULT_MINION_OPTS.copy()
        mock_data = {"fun": "foo.bar", "jid": 123}
        mock_jid_queue = [123]
        proxy_minion = salt.minion.ProxyMinion(
            mock_opts,
            jid_queue=copy.copy(mock_jid_queue),
            io_loop=salt.ext.tornado.ioloop.IOLoop(),
        )
        try:
            ret = proxy_minion._handle_decoded_payload(mock_data).result()
            self.assertEqual(proxy_minion.jid_queue, mock_jid_queue)
            self.assertIsNone(ret)
            self.assert_called_once(salt.minion._metaproxy_call)
        finally:
            proxy_minion.destroy()

    @slowTest
    def test_handle_payload_metaproxy_called(self):
        """
        Tests that when the _handle_payload function is called, _metaproxy_call is also called.
        """
        mock_opts = salt.config.DEFAULT_MINION_OPTS.copy()
        mock_data = {"fun": "foo.bar", "jid": 123}
        mock_jid_queue = [123]
        proxy_minion = salt.minion.ProxyMinion(
            mock_opts,
            jid_queue=copy.copy(mock_jid_queue),
            io_loop=salt.ext.tornado.ioloop.IOLoop(),
        )
        try:
            ret = proxy_minion._handle_decoded_payload(mock_data).result()
            self.assertEqual(proxy_minion.jid_queue, mock_jid_queue)
            self.assertIsNone(ret)
            self.assert_called_once(salt.minion._metaproxy_call)
        finally:
            proxy_minion.destroy()



# -*- coding: utf-8 -*-
"""
Test the grains module
"""

from __future__ import absolute_import, print_function, unicode_literals

import logging
import os
import pprint
import time

import pytest
from salt.ext.six.moves import range
from tests.support.case import ModuleCase
from tests.support.helpers import flaky, slowTest
from tests.support.unit import skipIf

log = logging.getLogger(__name__)


@pytest.mark.windows_whitelisted
class TestModulesGrains(ModuleCase):
    """
    Test the grains module
    """

    @slowTest
    def test_items(self):
        """
        grains.items
        """
        opts = self.minion_opts
        self.assertEqual(
            self.run_function("grains.items")["test_grain"],
            opts["grains"]["test_grain"],
        )

    @slowTest
    def test_item(self):
        """
        grains.item
        """
        opts = self.minion_opts
        self.assertEqual(
            self.run_function("grains.item", ["test_grain"])["test_grain"],
            opts["grains"]["test_grain"],
        )

    @slowTest
    def test_ls(self):
        """
        grains.ls
        """
        check_for = (
            "cpu_flags",
            "cpu_model",
            "cpuarch",
            "domain",
            "fqdn",
            "fqdns",
            "gid",
            "groupname",
            "host",
            "kernel",
            "kernelrelease",
            "kernelversion",
            "localhost",
            "mem_total",
            "num_cpus",
            "os",
            "os_family",
            "path",
            "pid",
            "ps",
            "pythonpath",
            "pythonversion",
            "saltpath",
            "saltversion",
            "uid",
            "username",
            "virtual",
        )
        lsgrains = self.run_function("grains.ls")
        os = self.run_function("grains.get", ["os"])
        for grain in check_for:
            if os == "Windows" and grain in ["cpu_flags", "gid", "groupname", "uid"]:
                continue
            self.assertTrue(grain in lsgrains)

    @skipIf(
        os.environ.get("TRAVIS_PYTHON_VERSION", None) is not None,
        "Travis environment can't keep up with salt refresh",
    )
    @slowTest
    def test_set_val(self):
        """
        test grains.set_val
        """
        self.assertEqual(
            self.run_function("grains.setval", ["setgrain", "grainval"]),
            {"setgrain": "grainval"},
        )
        time.sleep(5)
        ret = self.run_function("grains.item", ["setgrain"])
        if not ret:
            # Sleep longer, sometimes test systems get bogged down
            time.sleep(20)
            ret = self.run_function("grains.item", ["setgrain"])
        self.assertTrue(ret)

    @slowTest
    def test_get(self):
        """
        test grains.get
        """
        self.assertEqual(self.run_function("grains.get", ["level1:level2"]), "foo")

    @slowTest
    def test_get_core_grains(self):
        """
        test to ensure some core grains are returned
        """
        grains = ("os", "os_family", "osmajorrelease", "osrelease", "osfullname", "id")
        os = self.run_function("grains.get", ["os"])

        for grain in grains:
            get_grain = self.run_function("grains.get", [grain])
            log.debug("Value of '%s' grain: '%s'", grain, get_grain)
            if os == "Arch" and grain in ["osmajorrelease"]:
                self.assertEqual(get_grain, "")
                continue
            if os == "Windows" and grain in ["osmajorrelease"]:
                self.assertEqual(get_grain, "")
                continue

            self.assertTrue(get_grain)

    @slowTest
    def test_get_grains_int(self):
        """
        test to ensure int grains
        are returned as integers
        """
        grains = ["num_cpus", "mem_total", "num_gpus", "uid"]
        os = self.run_function("grains.get", ["os"])
        for grain in grains:
            get_grain = self.run_function("grains.get", [grain])
            if os == "Windows" and grain in ["uid"]:
                self.assertEqual(get_grain, "")
                continue
            self.assertIsInstance(
                get_grain, int, msg="grain: {0} is not an int or empty".format(grain)
            )


@pytest.mark.windows_whitelisted
class GrainsAppendTestCase(ModuleCase):
    """
    Tests written specifically for the grains.append function.
    """

    GRAIN_KEY = "salttesting-grain-key"
    GRAIN_VAL = "my-grain-val"

    def setUp(self):
        # Start off with an empty list
        self.run_function("grains.setval", [self.GRAIN_KEY, []])
        if not self.wait_for_grain(self.GRAIN_KEY, []):
            raise Exception("Failed to set grain")
        self.addCleanup(self.cleanup_grain)

    def cleanup_grain(self):
        self.run_function("grains.setval", [self.GRAIN_KEY, []])
        if not self.wait_for_grain(self.GRAIN_KEY, []):
            raise Exception("Failed to set grain")

    @slowTest
    def test_grains_append(self):
        """
        Tests the return of a simple grains.append call.
        """
        ret = self.run_function("grains.append", [self.GRAIN_KEY, self.GRAIN_VAL])
        self.assertEqual(ret[self.GRAIN_KEY], [self.GRAIN_VAL])

    @slowTest
    def test_grains_append_val_already_present(self):
        """
        Tests the return of a grains.append call when the value is already
        present in the grains list.
        """
        msg = "The val {0} was already in the list " "salttesting-grain-key".format(
            self.GRAIN_VAL
        )

        # First, make sure the test grain is present
        ret = self.run_function("grains.append", [self.GRAIN_KEY, self.GRAIN_VAL])
        self.assertEqual(ret[self.GRAIN_KEY], [self.GRAIN_VAL])

        # Now try to append again
        ret = self.run_function("grains.append", [self.GRAIN_KEY, self.GRAIN_VAL])
        self.assertTrue(self.wait_for_grain(self.GRAIN_KEY, [self.GRAIN_VAL]))
        if not ret or isinstance(ret, dict):
            # Sleep for a bit, sometimes the second "append" runs too quickly
            time.sleep(5)
            ret = self.run_function("grains.append", [self.GRAIN_KEY, self.GRAIN_VAL])

        assert msg == ret

    @flaky
    @slowTest
    def test_grains_append_val_is_list(self):
        """
        Tests the return of a grains.append call when val is passed in as a list.
        """
        # Start off with an empty list, don't know if the flaky decorator runs the setUp function or not...
        self.run_function("grains.setval", [self.GRAIN_KEY, []])
        second_grain = self.GRAIN_VAL + "-2"
        ret = self.run_function(
            "grains.append", [self.GRAIN_KEY, [self.GRAIN_VAL, second_grain]]
        )
        self.assertEqual(ret[self.GRAIN_KEY], [self.GRAIN_VAL, second_grain])

    @slowTest
    def test_grains_append_call_twice(self):
        """
        Tests the return of a grains.append call when the value is already present
        but also ensure the grain is not listed twice.
        """
        # First, add the test grain.
        append_1 = self.run_function("grains.append", [self.GRAIN_KEY, self.GRAIN_VAL])

        # Call the function again, which results in a string message, as tested in
        # test_grains_append_val_already_present above.
        append_2 = self.run_function("grains.append", [self.GRAIN_KEY, self.GRAIN_VAL])

        # Now make sure the grain doesn't show up twice.
        grains = self.run_function("grains.items")
        count = 0
        for grain in grains:
            if grain == self.GRAIN_KEY:
                count += 1

        # We should only have hit the grain key once.
        self.assertEqual(
            count,
            1,
            msg="Count did not match({}!=1) while looking for key '{}'.\nFirst append return:\n{}\nSecond append return:\n{}".format(
                count,
                self.GRAIN_KEY,
                pprint.pformat(append_1),
                pprint.pformat(append_2),
            ),
        )

    def wait_for_grain(self, key, val, timeout=60, sleep=0.3):
        start = time.time()
        while time.time() - start <= timeout:
            ret = self.run_function("grains.get", [key])
            if ret == val:
                return True
            time.sleep(sleep)
        return False

    @slowTest
    def test_grains_remove_add(self):
        second_grain = self.GRAIN_VAL + "-2"
        ret = self.run_function("grains.get", [self.GRAIN_KEY])
        self.assertEqual(ret, [])

        for i in range(10):
            ret = self.run_function("grains.setval", [self.GRAIN_KEY, []])
            self.assertEqual(ret[self.GRAIN_KEY], [])
            self.wait_for_grain(self.GRAIN_KEY, [])
            ret = self.run_function("grains.append", [self.GRAIN_KEY, self.GRAIN_VAL])
            self.assertEqual(ret[self.GRAIN_KEY], [self.GRAIN_VAL])
            self.assertEqual(ret[self.GRAIN_KEY], [self.GRAIN_VAL])

            ret = self.run_function("grains.setval", [self.GRAIN_KEY, []])
            self.wait_for_grain(self.GRAIN_KEY, [])
            self.assertTrue(self.wait_for_grain(self.GRAIN_KEY, []))
            ret = self.run_function(
                "grains.append", [self.GRAIN_KEY, [self.GRAIN_VAL, second_grain]]
            )
            self.assertEqual(ret[self.GRAIN_KEY], [self.GRAIN_VAL, second_grain])
            self.assertEqual(ret[self.GRAIN_KEY], [self.GRAIN_VAL, second_grain])



# -*- coding: utf-8 -*-
"""
:codeauthor: Shane Lee <slee@saltstack.com>
"""
# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import os
import sys

import salt.modules.cmdmod as cmdmod

# Import Salt Libs
import salt.modules.temp as temp
import salt.modules.win_file as win_file
import salt.utils.platform
import salt.utils.win_dacl as win_dacl
import salt.utils.win_functions
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase, skipIf

try:
    WIN_VER = sys.getwindowsversion().major
except AttributeError:
    WIN_VER = 0


class DummyStat(object):
    st_mode = 33188
    st_ino = 115331251
    st_dev = 44
    st_nlink = 1
    st_uid = 99200001
    st_gid = 99200001
    st_size = 41743
    st_atime = 1552661253
    st_mtime = 1552661253
    st_ctime = 1552661253


class WinFileTestCase(TestCase, LoaderModuleMockMixin):
    """
        Test cases for salt.modules.win_file
    """

    FAKE_RET = {"fake": "ret data"}
    if salt.utils.platform.is_windows():
        FAKE_PATH = os.sep.join(["C:", "path", "does", "not", "exist"])
    else:
        FAKE_PATH = os.sep.join(["path", "does", "not", "exist"])

    def setup_loader_modules(self):
        return {
            win_file: {
                "__utils__": {"dacl.set_perms": win_dacl.set_perms},
                "__salt__": {"cmd.run_stdout": cmdmod.run_stdout},
            }
        }

    def test_issue_43328_stats(self):
        """
        Make sure that a CommandExecutionError is raised if the file does NOT
        exist
        """
        with patch("os.path.exists", return_value=False):
            self.assertRaises(CommandExecutionError, win_file.stats, self.FAKE_PATH)

    def test_issue_43328_check_perms_no_ret(self):
        """
        Make sure that a CommandExecutionError is raised if the file does NOT
        exist
        """
        with patch("os.path.exists", return_value=False):
            self.assertRaises(
                CommandExecutionError, win_file.check_perms, self.FAKE_PATH
            )

    @skipIf(not salt.utils.platform.is_windows(), "Skip on Non-Windows systems")
    @skipIf(WIN_VER < 6, "Symlinks not supported on Vista an lower")
    def test_issue_52002_check_file_remove_symlink(self):
        """
        Make sure that directories including symlinks or symlinks can be removed
        """
        base = temp.dir(prefix="base-", parent=RUNTIME_VARS.TMP)
        target = os.path.join(base, "child 1", "target\\")
        symlink = os.path.join(base, "child 2", "link")
        try:
            # Create environment
            self.assertFalse(win_file.directory_exists(target))
            self.assertFalse(win_file.directory_exists(symlink))
            self.assertTrue(win_file.makedirs_(target))
            self.assertTrue(win_file.makedirs_(symlink))
            self.assertTrue(win_file.symlink(target, symlink))
            self.assertTrue(win_file.directory_exists(symlink))
            self.assertTrue(win_file.is_link(symlink))
            # Test removal of directory containing symlink
            self.assertTrue(win_file.remove(base))
            self.assertFalse(win_file.directory_exists(base))
        finally:
            if os.path.exists(base):
                win_file.remove(base)


@skipIf(not salt.utils.platform.is_windows(), "Skip on Non-Windows systems")
class WinFileCheckPermsTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for the check_perms function in salt.modules.win_file
    """

    temp_file = ""
    current_user = ""

    def setup_loader_modules(self):
        self.current_user = salt.utils.win_functions.get_current_user(False)
        return {
            win_file: {
                "__utils__": {
                    "dacl.check_perms": win_dacl.check_perms,
                    "dacl.set_perms": win_dacl.set_perms,
                }
            },
            win_dacl: {"__opts__": {"test": False}},
        }

    def setUp(self):
        self.temp_file = temp.file(parent=RUNTIME_VARS.TMP)
        win_dacl.set_owner(obj_name=self.temp_file, principal=self.current_user)
        win_dacl.set_inheritance(obj_name=self.temp_file, enabled=True)
        self.assertEqual(win_dacl.get_owner(obj_name=self.temp_file), self.current_user)

    def tearDown(self):
        os.remove(self.temp_file)

    def test_check_perms_set_owner_test_true(self):
        """
        Test setting the owner of a file with test=True
        """
        expected = {
            "comment": "",
            "changes": {"owner": "Administrators"},
            "name": self.temp_file,
            "result": None,
        }
        with patch.dict(win_dacl.__opts__, {"test": True}):
            ret = win_file.check_perms(
                path=self.temp_file, owner="Administrators", inheritance=None
            )
            self.assertDictEqual(expected, ret)

    def test_check_perms_set_owner(self):
        """
        Test setting the owner of a file
        """
        expected = {
            "comment": "",
            "changes": {"owner": "Administrators"},
            "name": self.temp_file,
            "result": True,
        }
        ret = win_file.check_perms(
            path=self.temp_file, owner="Administrators", inheritance=None
        )
        self.assertDictEqual(expected, ret)

    def test_check_perms_deny_test_true(self):
        """
        Test setting deny perms on a file with test=True
        """
        expected = {
            "comment": "",
            "changes": {"perms": {"Users": {"deny": "read_execute"}}},
            "name": self.temp_file,
            "result": None,
        }
        with patch.dict(win_dacl.__opts__, {"test": True}):
            ret = win_file.check_perms(
                path=self.temp_file,
                deny_perms={"Users": {"perms": "read_execute"}},
                inheritance=None,
            )
            self.assertDictEqual(expected, ret)

    def test_check_perms_deny(self):
        """
        Test setting deny perms on a file
        """
        expected = {
            "comment": "",
            "changes": {"perms": {"Users": {"deny": "read_execute"}}},
            "name": self.temp_file,
            "result": True,
        }
        ret = win_file.check_perms(
            path=self.temp_file,
            deny_perms={"Users": {"perms": "read_execute"}},
            inheritance=None,
        )
        self.assertDictEqual(expected, ret)

    def test_check_perms_grant_test_true(self):
        """
        Test setting grant perms on a file with test=True
        """
        expected = {
            "comment": "",
            "changes": {"perms": {"Users": {"grant": "read_execute"}}},
            "name": self.temp_file,
            "result": None,
        }
        with patch.dict(win_dacl.__opts__, {"test": True}):
            ret = win_file.check_perms(
                path=self.temp_file,
                grant_perms={"Users": {"perms": "read_execute"}},
                inheritance=None,
            )
            self.assertDictEqual(expected, ret)

    def test_check_perms_grant(self):
        """
        Test setting grant perms on a file
        """
        expected = {
            "comment": "",
            "changes": {"perms": {"Users": {"grant": "read_execute"}}},
            "name": self.temp_file,
            "result": True,
        }
        ret = win_file.check_perms(
            path=self.temp_file,
            grant_perms={"Users": {"perms": "read_execute"}},
            inheritance=None,
        )
        self.assertDictEqual(expected, ret)

    def test_check_perms_inheritance_false_test_true(self):
        """
        Test setting inheritance to False with test=True
        """
        expected = {
            "comment": "",
            "changes": {"inheritance": False},
            "name": self.temp_file,
            "result": None,
        }
        with patch.dict(win_dacl.__opts__, {"test": True}):
            ret = win_file.check_perms(path=self.temp_file, inheritance=False)
            self.assertDictEqual(expected, ret)

    def test_check_perms_inheritance_false(self):
        """
        Test setting inheritance to False
        """
        expected = {
            "comment": "",
            "changes": {"inheritance": False},
            "name": self.temp_file,
            "result": True,
        }
        ret = win_file.check_perms(path=self.temp_file, inheritance=False)
        self.assertDictEqual(expected, ret)

    def test_check_perms_inheritance_true(self):
        """
        Test setting inheritance to true when it's already true (default)
        """
        expected = {
            "comment": "",
            "changes": {},
            "name": self.temp_file,
            "result": True,
        }
        ret = win_file.check_perms(path=self.temp_file, inheritance=True)
        self.assertDictEqual(expected, ret)

    def test_check_perms_reset_test_true(self):
        """
        Test resetting perms with test=True. This shows minimal changes
        """
        # Turn off inheritance
        win_dacl.set_inheritance(obj_name=self.temp_file, enabled=False, clear=True)
        # Set some permissions
        win_dacl.set_permissions(
            obj_name=self.temp_file,
            principal="Administrator",
            permissions="full_control",
        )
        expected = {
            "comment": "",
            "changes": {
                "perms": {
                    "Administrators": {"grant": "full_control"},
                    "Users": {"grant": "read_execute"},
                },
                "remove_perms": {
                    "Administrator": {
                        "grant": {
                            "applies to": "Not Inherited (file)",
                            "permissions": "Full control",
                        }
                    }
                },
            },
            "name": self.temp_file,
            "result": None,
        }
        with patch.dict(win_dacl.__opts__, {"test": True}):
            ret = win_file.check_perms(
                path=self.temp_file,
                grant_perms={
                    "Users": {"perms": "read_execute"},
                    "Administrators": {"perms": "full_control"},
                },
                inheritance=False,
                reset=True,
            )
            self.assertDictEqual(expected, ret)

    def test_check_perms_reset(self):
        """
        Test resetting perms on a File
        """
        # Turn off inheritance
        win_dacl.set_inheritance(obj_name=self.temp_file, enabled=False, clear=True)
        # Set some permissions
        win_dacl.set_permissions(
            obj_name=self.temp_file,
            principal="Administrator",
            permissions="full_control",
        )
        expected = {
            "comment": "",
            "changes": {
                "perms": {
                    "Administrators": {"grant": "full_control"},
                    "Users": {"grant": "read_execute"},
                },
                "remove_perms": {
                    "Administrator": {
                        "grant": {
                            "applies to": "Not Inherited (file)",
                            "permissions": "Full control",
                        }
                    }
                },
            },
            "name": self.temp_file,
            "result": True,
        }
        ret = win_file.check_perms(
            path=self.temp_file,
            grant_perms={
                "Users": {"perms": "read_execute"},
                "Administrators": {"perms": "full_control"},
            },
            inheritance=False,
            reset=True,
        )
        self.assertDictEqual(expected, ret)

    def test_stat(self):
        with patch("os.path.exists", MagicMock(return_value=True)), patch(
            "salt.modules.win_file._resolve_symlink",
            MagicMock(side_effect=lambda path: path),
        ), patch("salt.modules.win_file.get_uid", MagicMock(return_value=1)), patch(
            "salt.modules.win_file.uid_to_user", MagicMock(return_value="dummy")
        ), patch(
            "salt.modules.win_file.get_pgid", MagicMock(return_value=1)
        ), patch(
            "salt.modules.win_file.gid_to_group", MagicMock(return_value="dummy")
        ), patch(
            "os.stat", MagicMock(return_value=DummyStat())
        ):
            ret = win_file.stats("dummy", None, True)
            self.assertEqual(ret["mode"], "0644")
            self.assertEqual(ret["type"], "file")



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.mac_xattr as xattr

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class XAttrTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {xattr: {}}

    def test_exists_not(self):
        """
            Test adding an attribute when it doesn't exist
        """
        with patch("os.path.exists") as exists_mock:
            expected = {
                "changes": {"key": "value"},
                "comment": "",
                "name": "/path/to/file",
                "result": True,
            }

            exists_mock.return_value = True

            list_mock = MagicMock(return_value={"other.id": "value2"})
            write_mock = MagicMock()
            with patch.dict(
                xattr.__salt__, {"xattr.list": list_mock, "xattr.write": write_mock}
            ):
                out = xattr.exists("/path/to/file", ["key=value"])

                list_mock.assert_called_once_with("/path/to/file")
                write_mock.assert_called_once_with(
                    "/path/to/file", "key", "value", False
                )
                self.assertEqual(out, expected)

    def test_exists_change(self):
        """
            Test changing and attribute value
        """
        with patch("os.path.exists") as exists_mock:
            expected = {
                "changes": {"key": "other_value"},
                "comment": "",
                "name": "/path/to/file",
                "result": True,
            }

            exists_mock.return_value = True

            list_mock = MagicMock(return_value={"key": "value"})
            write_mock = MagicMock()
            with patch.dict(
                xattr.__salt__, {"xattr.list": list_mock, "xattr.write": write_mock}
            ):
                out = xattr.exists("/path/to/file", ["key=other_value"])

                list_mock.assert_called_once_with("/path/to/file")
                write_mock.assert_called_once_with(
                    "/path/to/file", "key", "other_value", False
                )
                self.assertEqual(out, expected)

    def test_exists_already(self):
        """
            Test that having the same value does nothing
        """
        with patch("os.path.exists") as exists_mock:
            expected = {
                "changes": {},
                "comment": "All values existed correctly.",
                "name": "/path/to/file",
                "result": True,
            }

            exists_mock.return_value = True

            list_mock = MagicMock(return_value={"key": "value"})
            write_mock = MagicMock()
            with patch.dict(
                xattr.__salt__, {"xattr.list": list_mock, "xattr.write": write_mock}
            ):
                out = xattr.exists("/path/to/file", ["key=value"])

                list_mock.assert_called_once_with("/path/to/file")
                assert not write_mock.called
                self.assertEqual(out, expected)

    def test_delete(self):
        """
            Test deleting an attribute from a file
        """
        with patch("os.path.exists") as exists_mock:
            expected = {
                "changes": {"key": "delete"},
                "comment": "",
                "name": "/path/to/file",
                "result": True,
            }

            exists_mock.return_value = True

            list_mock = MagicMock(return_value={"key": "value2"})
            delete_mock = MagicMock()
            with patch.dict(
                xattr.__salt__, {"xattr.list": list_mock, "xattr.delete": delete_mock}
            ):
                out = xattr.delete("/path/to/file", ["key"])

                list_mock.assert_called_once_with("/path/to/file")
                delete_mock.assert_called_once_with("/path/to/file", "key")
                self.assertEqual(out, expected)

    def test_delete_not(self):
        """
            Test deleting an attribute that doesn't exist from a file
        """
        with patch("os.path.exists") as exists_mock:
            expected = {
                "changes": {},
                "comment": "All attributes were already deleted.",
                "name": "/path/to/file",
                "result": True,
            }

            exists_mock.return_value = True

            list_mock = MagicMock(return_value={"other.key": "value2"})
            delete_mock = MagicMock()
            with patch.dict(
                xattr.__salt__, {"xattr.list": list_mock, "xattr.delete": delete_mock}
            ):
                out = xattr.delete("/path/to/file", ["key"])

                list_mock.assert_called_once_with("/path/to/file")
                assert not delete_mock.called
                self.assertEqual(out, expected)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import os

import salt.exceptions

# Import salt libs
import salt.modules.vagrant as vagrant
import salt.utils.platform

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase

TEMP_DATABASE_FILE = "/tmp/salt-tests-tmpdir/test_vagrant.sqlite"


class VagrantTestCase(TestCase, LoaderModuleMockMixin):
    """
    Unit TestCase for the salt.modules.vagrant module.
    """

    LOCAL_OPTS = {
        "extension_modules": "",
        "vagrant_sdb_data": {
            "driver": "sqlite3",
            "database": TEMP_DATABASE_FILE,
            "table": "sdb",
            "create_table": True,
        },
    }

    def setup_loader_modules(self):
        vagrant_globals = {
            "__opts__": self.LOCAL_OPTS,
        }
        return {vagrant: vagrant_globals}

    def test_vagrant_get_vm_info_not_found(self):
        mock_sdb = MagicMock(return_value=None)
        with patch.dict(vagrant.__utils__, {"sdb.sdb_get": mock_sdb}):
            with self.assertRaises(salt.exceptions.SaltInvocationError):
                vagrant.get_vm_info("thisNameDoesNotExist")

    def test_vagrant_init_positional(self):
        path_nowhere = os.path.join(os.sep, "tmp", "nowhere")
        if salt.utils.platform.is_windows():
            path_nowhere = "c:{0}".format(path_nowhere)
        mock_sdb = MagicMock(return_value=None)
        with patch.dict(vagrant.__utils__, {"sdb.sdb_set": mock_sdb}):
            resp = vagrant.init(
                "test1",
                path_nowhere,
                "onetest",
                "nobody",
                False,
                "french",
                {"different": "very"},
            )
            self.assertTrue(resp.startswith("Name test1 defined"))
            expected = dict(
                name="test1",
                cwd=path_nowhere,
                machine="onetest",
                runas="nobody",
                vagrant_provider="french",
                different="very",
            )
            mock_sdb.assert_called_with(
                "sdb://vagrant_sdb_data/onetest?{0}".format(path_nowhere),
                "test1",
                self.LOCAL_OPTS,
            )
            mock_sdb.assert_any_call(
                "sdb://vagrant_sdb_data/test1", expected, self.LOCAL_OPTS
            )

    def test_vagrant_get_vm_info(self):
        testdict = {"testone": "one", "machine": "two"}
        mock_sdb = MagicMock(return_value=testdict)
        with patch.dict(vagrant.__utils__, {"sdb.sdb_get": mock_sdb}):
            resp = vagrant.get_vm_info("test1")
            self.assertEqual(resp, testdict)

    def test_vagrant_init_dict(self):
        testdict = dict(
            cwd="/tmp/anywhere",
            machine="twotest",
            runas="somebody",
            vagrant_provider="english",
        )
        expected = testdict.copy()
        expected["name"] = "test2"
        mock_sdb = MagicMock(return_value=None)
        with patch.dict(vagrant.__utils__, {"sdb.sdb_set": mock_sdb}):
            vagrant.init("test2", vm=testdict)
            mock_sdb.assert_any_call(
                "sdb://vagrant_sdb_data/test2", expected, self.LOCAL_OPTS
            )

    def test_vagrant_init_arg_override(self):
        testdict = dict(
            cwd="/tmp/there",
            machine="treetest",
            runas="anybody",
            vagrant_provider="spansh",
        )
        mock_sdb = MagicMock(return_value=None)
        with patch.dict(vagrant.__utils__, {"sdb.sdb_set": mock_sdb}):
            vagrant.init(
                "test3",
                cwd="/tmp",
                machine="threetest",
                runas="him",
                vagrant_provider="polish",
                vm=testdict,
            )
            expected = dict(
                name="test3",
                cwd="/tmp",
                machine="threetest",
                runas="him",
                vagrant_provider="polish",
            )
            mock_sdb.assert_any_call(
                "sdb://vagrant_sdb_data/test3", expected, self.LOCAL_OPTS
            )

    def test_vagrant_get_ssh_config_fails(self):
        mock_sdb = MagicMock(return_value=None)
        with patch.dict(vagrant.__utils__, {"sdb.sdb_set": mock_sdb}):
            mock_sdb = MagicMock(return_value={})
            with patch.dict(vagrant.__utils__, {"sdb.sdb_get": mock_sdb}):
                vagrant.init("test3", cwd="/tmp")
                with self.assertRaises(salt.exceptions.SaltInvocationError):
                    vagrant.get_ssh_config("test3")  # has not been started

    def test_vagrant_destroy(self):
        path_mydir = os.path.join(os.sep, "my", "dir")
        if salt.utils.platform.is_windows():
            path_mydir = "c:{0}".format(path_mydir)
        mock_cmd = MagicMock(return_value={"retcode": 0})
        with patch.dict(vagrant.__salt__, {"cmd.run_all": mock_cmd}):
            mock_sdb = MagicMock(return_value=None)
            with patch.dict(vagrant.__utils__, {"sdb.sdb_delete": mock_sdb}):
                mock_sdb_get = MagicMock(
                    return_value={"machine": "macfour", "cwd": path_mydir}
                )
                with patch.dict(vagrant.__utils__, {"sdb.sdb_get": mock_sdb_get}):
                    self.assertTrue(vagrant.destroy("test4"))
                    mock_sdb.assert_any_call(
                        "sdb://vagrant_sdb_data/macfour?{0}".format(path_mydir),
                        self.LOCAL_OPTS,
                    )
                    mock_sdb.assert_any_call(
                        "sdb://vagrant_sdb_data/test4", self.LOCAL_OPTS
                    )
                    cmd = "vagrant destroy -f macfour"
                    mock_cmd.assert_called_with(
                        cmd, runas=None, cwd=path_mydir, output_loglevel="info"
                    )

    def test_vagrant_start(self):
        mock_cmd = MagicMock(return_value={"retcode": 0})
        with patch.dict(vagrant.__salt__, {"cmd.run_all": mock_cmd}):
            mock_sdb_get = MagicMock(
                return_value={
                    "machine": "five",
                    "cwd": "/the/dir",
                    "runas": "me",
                    "vagrant_provider": "him",
                }
            )
            with patch.dict(vagrant.__utils__, {"sdb.sdb_get": mock_sdb_get}):
                self.assertTrue(vagrant.start("test5"))
                cmd = "vagrant up five --provider=him"
                mock_cmd.assert_called_with(
                    cmd, runas="me", cwd="/the/dir", output_loglevel="info"
                )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.znc as znc

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class ZncTestCase(TestCase, LoaderModuleMockMixin):
    """
    TestCase for salt.modules.znc
    """

    def setup_loader_modules(self):
        return {znc: {}}

    # 'buildmod' function tests: 1

    def test_buildmod(self):
        """
        Tests build module using znc-buildmod
        """
        with patch("os.path.exists", MagicMock(return_value=False)):
            self.assertEqual(
                znc.buildmod("modules.cpp"),
                "Error: The file (modules.cpp) does not exist.",
            )

    def test_buildmod_module(self):
        """
        Tests build module using znc-buildmod
        """
        mock = MagicMock(return_value="SALT")
        with patch.dict(znc.__salt__, {"cmd.run": mock}), patch(
            "os.path.exists", MagicMock(return_value=True)
        ):
            self.assertEqual(znc.buildmod("modules.cpp"), "SALT")

    # 'dumpconf' function tests: 1

    def test_dumpconf(self):
        """
        Tests write the active configuration state to config file
        """
        mock = MagicMock(return_value="SALT")
        with patch.dict(znc.__salt__, {"ps.pkill": mock}), patch.object(
            znc, "signal", MagicMock()
        ):
            self.assertEqual(znc.dumpconf(), "SALT")

    # 'rehashconf' function tests: 1

    def test_rehashconf(self):
        """
        Tests rehash the active configuration state from config file
        """
        mock = MagicMock(return_value="SALT")
        with patch.dict(znc.__salt__, {"ps.pkill": mock}), patch.object(
            znc, "signal", MagicMock()
        ):
            self.assertEqual(znc.rehashconf(), "SALT")

    # 'version' function tests: 1

    def test_version(self):
        """
        Tests return server version from znc --version
        """
        mock = MagicMock(return_value="ZNC 1.2 - http://znc.in")
        with patch.dict(znc.__salt__, {"cmd.run": mock}):
            self.assertEqual(znc.version(), "ZNC 1.2")



# -*- coding: utf-8 -*-
"""
    :codeauthor: :email:`Megan Wilhite (mwilhite@saltstack.com)`


    tests.unit.returners.test_syslog_return
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import salt libs
import salt.returners.syslog_return as syslog

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf


class SyslogReturnerTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test Syslog returner
    """

    def setup_loader_modules(self):
        return {syslog: {}}

    @skipIf(not syslog.HAS_SYSLOG, "Skip when syslog not installed")
    def test_syslog_returner_unicode(self):
        """
        test syslog returner with unicode
        """
        ret = {
            "fun_args": [],
            "jid": "20180713160901624786",
            "return": True,
            "retcode": 0,
            "success": True,
            "fun": "test.ping",
            "id": "02e10e971a30",
        }
        opts = {
            "level": "LOG_INFO",
            "options": [],
            "facility": "LOG_USER",
            "tag": "salt-minion",
        }

        with patch(
            "salt.returners.syslog_return._get_options", MagicMock(return_value=opts)
        ):
            try:
                syslog.returner(ret)
            except Exception as e:  # pylint: disable=broad-except
                self.fail("syslog.returner() failed with exception: {0}".format(e))



# -*- coding: utf-8 -*-
"""
Test the win_wua execution module
"""
# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.win_wua as win_wua
import salt.utils.platform
import salt.utils.win_update

# Import Salt Testing Libs
from tests.support.mock import patch
from tests.support.unit import TestCase, skipIf

UPDATES_LIST = {
    "ca3bb521-a8ea-4e26-a563-2ad6e3108b9a": {"KBs": ["KB4481252"]},
    "07609d43-d518-4e77-856e-d1b316d1b8a8": {"KBs": ["KB925673"]},
    "fbaa5360-a440-49d8-a3b6-0c4fc7ecaa19": {"KBs": ["KB4481252"]},
    "a873372b-7a5c-443c-8022-cd59a550bef4": {"KBs": ["KB3193497"]},
    "14075cbe-822e-4004-963b-f50e08d45563": {"KBs": ["KB4540723"]},
    "d931e99c-4dda-4d39-9905-0f6a73f7195f": {"KBs": ["KB3193497"]},
    "afda9e11-44a0-4602-9e9b-423af11ecaed": {"KBs": ["KB4541329"]},
    "a0f997b1-1abe-4a46-941f-b37f732f9fbd": {"KBs": ["KB3193497"]},
    "eac02b09-d745-4891-b80f-400e0e5e4b6d": {"KBs": ["KB4052623"]},
    "0689e74b-54d1-4f55-a916-96e3c737db90": {"KBs": ["KB890830"]},
}
UPDATES_SUMMARY = {"Installed": 10}


class Updates(object):
    @staticmethod
    def list():
        return UPDATES_LIST

    @staticmethod
    def summary():
        return UPDATES_SUMMARY


@skipIf(not salt.utils.platform.is_windows(), "System is not Windows")
class WinWuaInstalledTestCase(TestCase):
    """
    Test the functions in the win_wua.installed function
    """

    def test_installed(self):
        """
        Test installed function default
        """
        expected = UPDATES_LIST
        with patch("salt.utils.winapi.Com", autospec=True), patch(
            "win32com.client.Dispatch", autospec=True
        ), patch.object(
            salt.utils.win_update.WindowsUpdateAgent, "refresh", autospec=True
        ), patch.object(
            salt.utils.win_update, "Updates", autospec=True, return_value=Updates()
        ):
            result = win_wua.installed()
            self.assertDictEqual(result, expected)

    def test_installed_summary(self):
        """
        Test installed function with summary=True
        """
        expected = UPDATES_SUMMARY
        # Remove all updates that are not installed
        with patch("salt.utils.winapi.Com", autospec=True), patch(
            "win32com.client.Dispatch", autospec=True
        ), patch.object(
            salt.utils.win_update.WindowsUpdateAgent, "refresh", autospec=True
        ), patch.object(
            salt.utils.win_update, "Updates", autospec=True, return_value=Updates()
        ):
            result = win_wua.installed(summary=True)
            self.assertDictEqual(result, expected)

    def test_installed_kbs_only(self):
        """
        Test installed function with kbs_only=True
        """
        expected = set()
        for update in UPDATES_LIST:
            expected.update(UPDATES_LIST[update]["KBs"])
        expected = sorted(expected)
        # Remove all updates that are not installed
        with patch("salt.utils.winapi.Com", autospec=True), patch(
            "win32com.client.Dispatch", autospec=True
        ), patch.object(
            salt.utils.win_update.WindowsUpdateAgent, "refresh", autospec=True
        ), patch.object(
            salt.utils.win_update, "Updates", autospec=True, return_value=Updates()
        ):
            result = win_wua.installed(kbs_only=True)
            self.assertListEqual(result, expected)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    tests.unit.modules.alternatives_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import salt libs
import salt.modules.alternatives as alternatives
from tests.support.helpers import TstSuiteLoggingHandler
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch

# Import Salt Testing libs
from tests.support.unit import TestCase


class AlternativesTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {alternatives: {}}

    def test_display(self):
        with patch.dict(alternatives.__grains__, {"os_family": "RedHat"}):
            mock = MagicMock(return_value={"retcode": 0, "stdout": "salt"})
            with patch.dict(alternatives.__salt__, {"cmd.run_all": mock}):
                solution = alternatives.display("better-world")
                self.assertEqual("salt", solution)
                mock.assert_called_once_with(
                    ["alternatives", "--display", "better-world"],
                    ignore_retcode=True,
                    python_shell=False,
                )

        with patch.dict(alternatives.__grains__, {"os_family": "Suse"}):
            mock = MagicMock(return_value={"retcode": 0, "stdout": "undoubtedly-salt"})
            with patch.dict(alternatives.__salt__, {"cmd.run_all": mock}):
                solution = alternatives.display("better-world")
                self.assertEqual("undoubtedly-salt", solution)
                mock.assert_called_once_with(
                    ["update-alternatives", "--display", "better-world"],
                    ignore_retcode=True,
                    python_shell=False,
                )

        with patch.dict(alternatives.__grains__, {"os_family": "RedHat"}):
            mock = MagicMock(
                return_value={"retcode": 1, "stdout": "salt-out", "stderr": "salt-err"}
            )
            with patch.dict(alternatives.__salt__, {"cmd.run_all": mock}):
                solution = alternatives.display("better-world")
                self.assertEqual("salt-err", solution)
                mock.assert_called_once_with(
                    ["alternatives", "--display", "better-world"],
                    ignore_retcode=True,
                    python_shell=False,
                )

    def test_show_current(self):
        mock = MagicMock(return_value="/etc/alternatives/salt")
        with patch("salt.utils.path.readlink", mock):
            ret = alternatives.show_current("better-world")
            self.assertEqual("/etc/alternatives/salt", ret)
            mock.assert_called_once_with("/etc/alternatives/better-world")

            with TstSuiteLoggingHandler() as handler:
                mock.side_effect = OSError("Hell was not found!!!")
                self.assertFalse(alternatives.show_current("hell"))
                mock.assert_called_with("/etc/alternatives/hell")
                self.assertIn(
                    "ERROR:alternative: hell does not exist", handler.messages
                )

    def test_check_installed(self):
        mock = MagicMock(return_value="/etc/alternatives/salt")
        with patch("salt.utils.path.readlink", mock):
            self.assertTrue(
                alternatives.check_installed("better-world", "/etc/alternatives/salt")
            )
            mock.return_value = False
            self.assertFalse(
                alternatives.check_installed("help", "/etc/alternatives/salt")
            )

    def test_install(self):
        with patch.dict(alternatives.__grains__, {"os_family": "RedHat"}):
            mock = MagicMock(return_value={"retcode": 0, "stdout": "salt"})
            with patch.dict(alternatives.__salt__, {"cmd.run_all": mock}):
                solution = alternatives.install(
                    "better-world", "/usr/bin/better-world", "/usr/bin/salt", 100
                )
                self.assertEqual("salt", solution)
                mock.assert_called_once_with(
                    [
                        "alternatives",
                        "--install",
                        "/usr/bin/better-world",
                        "better-world",
                        "/usr/bin/salt",
                        "100",
                    ],
                    python_shell=False,
                )

        with patch.dict(alternatives.__grains__, {"os_family": "Debian"}):
            mock = MagicMock(return_value={"retcode": 0, "stdout": "salt"})
            with patch.dict(alternatives.__salt__, {"cmd.run_all": mock}):
                solution = alternatives.install(
                    "better-world", "/usr/bin/better-world", "/usr/bin/salt", 100
                )
                self.assertEqual("salt", solution)
                mock.assert_called_once_with(
                    [
                        "update-alternatives",
                        "--install",
                        "/usr/bin/better-world",
                        "better-world",
                        "/usr/bin/salt",
                        "100",
                    ],
                    python_shell=False,
                )

        with patch.dict(alternatives.__grains__, {"os_family": "RedHat"}):
            mock = MagicMock(
                return_value={"retcode": 1, "stdout": "salt-out", "stderr": "salt-err"}
            )
            with patch.dict(alternatives.__salt__, {"cmd.run_all": mock}):
                ret = alternatives.install(
                    "better-world", "/usr/bin/better-world", "/usr/bin/salt", 100
                )
                self.assertEqual("salt-err", ret)
                mock.assert_called_once_with(
                    [
                        "alternatives",
                        "--install",
                        "/usr/bin/better-world",
                        "better-world",
                        "/usr/bin/salt",
                        "100",
                    ],
                    python_shell=False,
                )

    def test_remove(self):
        with patch.dict(alternatives.__grains__, {"os_family": "RedHat"}):
            mock = MagicMock(return_value={"retcode": 0, "stdout": "salt"})
            with patch.dict(alternatives.__salt__, {"cmd.run_all": mock}):
                solution = alternatives.remove("better-world", "/usr/bin/better-world",)
                self.assertEqual("salt", solution)
                mock.assert_called_once_with(
                    [
                        "alternatives",
                        "--remove",
                        "better-world",
                        "/usr/bin/better-world",
                    ],
                    python_shell=False,
                )

        with patch.dict(alternatives.__grains__, {"os_family": "Debian"}):
            mock = MagicMock(return_value={"retcode": 0, "stdout": "salt"})
            with patch.dict(alternatives.__salt__, {"cmd.run_all": mock}):
                solution = alternatives.remove("better-world", "/usr/bin/better-world",)
                self.assertEqual("salt", solution)
                mock.assert_called_once_with(
                    [
                        "update-alternatives",
                        "--remove",
                        "better-world",
                        "/usr/bin/better-world",
                    ],
                    python_shell=False,
                )

        with patch.dict(alternatives.__grains__, {"os_family": "RedHat"}):
            mock = MagicMock(
                return_value={"retcode": 1, "stdout": "salt-out", "stderr": "salt-err"}
            )
            with patch.dict(alternatives.__salt__, {"cmd.run_all": mock}):
                solution = alternatives.remove("better-world", "/usr/bin/better-world",)
                self.assertEqual("salt-err", solution)
                mock.assert_called_once_with(
                    [
                        "alternatives",
                        "--remove",
                        "better-world",
                        "/usr/bin/better-world",
                    ],
                    python_shell=False,
                )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Andrew Colin Kissa <andrew@topdog.za.net>
"""
from __future__ import absolute_import, print_function, unicode_literals

import salt.states.postgres_privileges as postgres_privileges
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class PostgresPrivilegesTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.postgres_privileges
    """

    def setup_loader_modules(self):
        return {postgres_privileges: {}}

    def setUp(self):
        """
        Setup data for the tests
        """
        self.table_name = "awl"
        self.group_name = "admins"
        self.name = "baruwa"
        self.ret = {"name": self.name, "changes": {}, "result": False, "comment": ""}
        self.mock_true = MagicMock(return_value=True)
        self.mock_false = MagicMock(return_value=False)

    def tearDown(self):
        del self.ret
        del self.mock_true
        del self.mock_false

    def test_present_table(self):
        """
        Test present
        """
        with patch.dict(
            postgres_privileges.__salt__, {"postgres.has_privileges": self.mock_true}
        ):
            comt = "The requested privilege(s) are already set"
            self.ret.update({"comment": comt, "result": True})
            self.assertDictEqual(
                postgres_privileges.present(self.name, self.table_name, "table"),
                self.ret,
            )

        with patch.dict(
            postgres_privileges.__salt__,
            {
                "postgres.has_privileges": self.mock_false,
                "postgres.privileges_grant": self.mock_true,
            },
        ):
            with patch.dict(postgres_privileges.__opts__, {"test": True}):
                comt = ("The privilege(s): {0} are" " set to be granted to {1}").format(
                    "ALL", self.name
                )
                self.ret.update({"comment": comt, "result": None})
                self.assertDictEqual(
                    postgres_privileges.present(
                        self.name, self.table_name, "table", privileges=["ALL"]
                    ),
                    self.ret,
                )

            with patch.dict(postgres_privileges.__opts__, {"test": False}):
                comt = ("The privilege(s): {0} have " "been granted to {1}").format(
                    "ALL", self.name
                )
                self.ret.update(
                    {"comment": comt, "result": True, "changes": {"baruwa": "Present"}}
                )
                self.assertDictEqual(
                    postgres_privileges.present(
                        self.name, self.table_name, "table", privileges=["ALL"]
                    ),
                    self.ret,
                )

    def test_present_group(self):
        """
        Test present group
        """
        with patch.dict(
            postgres_privileges.__salt__,
            {
                "postgres.has_privileges": self.mock_false,
                "postgres.privileges_grant": self.mock_true,
            },
        ):
            with patch.dict(postgres_privileges.__opts__, {"test": True}):
                comt = ("The privilege(s): {0} are" " set to be granted to {1}").format(
                    self.group_name, self.name
                )
                self.ret.update({"comment": comt, "result": None})
                self.assertDictEqual(
                    postgres_privileges.present(self.name, self.group_name, "group"),
                    self.ret,
                )

            with patch.dict(postgres_privileges.__opts__, {"test": False}):
                comt = ("The privilege(s): {0} have " "been granted to {1}").format(
                    self.group_name, self.name
                )
                self.ret.update(
                    {"comment": comt, "result": True, "changes": {"baruwa": "Present"}}
                )
                self.assertDictEqual(
                    postgres_privileges.present(self.name, self.group_name, "group"),
                    self.ret,
                )

    def test_absent_table(self):
        """
        Test absent
        """
        with patch.dict(
            postgres_privileges.__salt__, {"postgres.has_privileges": self.mock_false}
        ):
            with patch.dict(postgres_privileges.__opts__, {"test": True}):
                comt = "The requested privilege(s)" " are not set so cannot be revoked"
                self.ret.update({"comment": comt, "result": True})
                self.assertDictEqual(
                    postgres_privileges.absent(self.name, self.table_name, "table"),
                    self.ret,
                )

        with patch.dict(
            postgres_privileges.__salt__,
            {
                "postgres.has_privileges": self.mock_true,
                "postgres.privileges_revoke": self.mock_true,
            },
        ):
            with patch.dict(postgres_privileges.__opts__, {"test": True}):
                comt = (
                    "The privilege(s): {0} are" " set to be revoked from {1}"
                ).format("ALL", self.name)
                self.ret.update({"comment": comt, "result": None})
                self.assertDictEqual(
                    postgres_privileges.absent(
                        self.name, self.table_name, "table", privileges=["ALL"]
                    ),
                    self.ret,
                )

            with patch.dict(postgres_privileges.__opts__, {"test": False}):
                comt = ("The privilege(s): {0} have " "been revoked from {1}").format(
                    "ALL", self.name
                )
                self.ret.update(
                    {"comment": comt, "result": True, "changes": {"baruwa": "Absent"}}
                )
                self.assertDictEqual(
                    postgres_privileges.absent(
                        self.name, self.table_name, "table", privileges=["ALL"]
                    ),
                    self.ret,
                )

    def test_absent_group(self):
        """
        Test absent group
        """
        with patch.dict(
            postgres_privileges.__salt__,
            {
                "postgres.has_privileges": self.mock_true,
                "postgres.privileges_revoke": self.mock_true,
            },
        ):
            with patch.dict(postgres_privileges.__opts__, {"test": True}):
                comt = (
                    "The privilege(s): {0} are" " set to be revoked from {1}"
                ).format(self.group_name, self.name)
                self.ret.update({"comment": comt, "result": None})
                self.assertDictEqual(
                    postgres_privileges.absent(self.name, self.group_name, "group"),
                    self.ret,
                )

            with patch.dict(postgres_privileges.__opts__, {"test": False}):
                comt = ("The privilege(s): {0} have " "been revoked from {1}").format(
                    self.group_name, self.name
                )
                self.ret.update(
                    {"comment": comt, "result": True, "changes": {"baruwa": "Absent"}}
                )
                self.assertDictEqual(
                    postgres_privileges.absent(self.name, self.group_name, "group"),
                    self.ret,
                )



# -*- coding: utf-8 -*-
"""
    :codeauthor: :email:`Jakub Sliva <jakub.sliva@ultimum.io>`
"""

# Import Python Libs
from __future__ import absolute_import, unicode_literals

import salt.states.zabbix_template as zabbix_template

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase

INPUT_PARAMS = {"applications": [{"name": "Ceph OSD"}]}

DEFINED_OBJ = {
    "macros": [{"macro": "{$CEPH_CLUSTER_NAME}", "value": "ceph"}],
    "host": "A Testing Template",
    "hosts": [{"hostid": "10112"}, {"hostid": "10113"}],
    "description": "Template for Ceph nodes",
    "groups": [{"groupid": "1"}],
}

DEFINED_C_LIST_SUBS = {
    "applications": [{"name": "Ceph OSD"}],
    "graphs": [],
    "triggers": [],
    "items": [],
    "httpTests": [],
    "screens": [],
    "gitems": [],
    "discoveries": [],
}

SUBSTITUTE_PARAMS_CREATE = [
    DEFINED_OBJ,
    [],
    DEFINED_C_LIST_SUBS["applications"],
    [],
    [],
    [],
    [],
    [],
    [],
    [],
    [],
]

EXISTING_OBJ = [
    {
        "available": "0",
        "tls_connect": "1",
        "maintenance_type": "0",
        "groups": [{"groupid": "1"}],
        "macros": [
            {
                "macro": "{$CEPH_CLUSTER_NAME}",
                "hostmacroid": "60",
                "hostid": "10206",
                "value": "ceph",
            }
        ],
        "hosts": [{"hostid": "10112"}, {"hostid": "10113"}],
        "status": "3",
        "description": "Template for Ceph nodes",
        "host": "A Testing Template",
        "disable_until": "0",
        "templateid": "10206",
        "name": "A Testing Template",
    }
]

SUBSTITUTE_PARAMS_EXISTS = [
    DEFINED_OBJ,
    EXISTING_OBJ[0],
    [],
    [],
    [],
    [],
    [],
    [],
    [],
    [],
]

EXISTING_OBJ_DIFF = [
    {
        "groups": [{"groupid": "1"}],
        "macros": [
            {
                "macro": "{$CEPH_CLUSTER_NAME}",
                "hostmacroid": "60",
                "hostid": "10206",
                "value": "ceph",
            }
        ],
        "hosts": [{"hostid": "10112"}, {"hostid": "10113"}],
        "status": "3",
        "templateid": "10206",
        "name": "A Testing Template",
    }
]

SUBSTITUTE_PARAMS_UPDATE = [
    DEFINED_OBJ,
    EXISTING_OBJ_DIFF[0],
    [],
    [],
    [],
    [],
    [],
    [],
    [],
    [],
]

DIFF_PARAMS = {"old": {}, "new": {"macros": [], "templateid": "10206"}}


class ZabbixTemplateTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.zabbix
    """

    def setup_loader_modules(self):
        return {zabbix_template: {}}

    @patch("salt.states.zabbix_template.CHANGE_STACK", [])
    def test_present_create(self):
        """
        Test to ensure that named template is created
        """
        name = "A Testing Template"
        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        def side_effect_run_query(*args):
            """
            Differentiate between __salt__ exec module function calls with different parameters.
            """
            if args[0] == "template.get":
                return []
            elif args[0] == "template.create":
                return {"templateids": ["10206"]}
            elif args[0] == "application.get":
                return []
            elif args[0] == "application.create":
                return {"applicationids": ["701"]}

        with patch.dict(zabbix_template.__opts__, {"test": False}):
            with patch.dict(
                zabbix_template.__salt__,
                {
                    "zabbix.get_zabbix_id_mapper": MagicMock(
                        return_value={"template": "templateid"}
                    ),
                    "zabbix.substitute_params": MagicMock(
                        side_effect=SUBSTITUTE_PARAMS_CREATE
                    ),
                    "zabbix.run_query": MagicMock(side_effect=side_effect_run_query),
                    "zabbix.compare_params": MagicMock(return_value={}),
                },
            ):
                ret["result"] = True
                ret["comment"] = 'Zabbix Template "{0}" created.'.format(name)
                ret["changes"] = {
                    name: {
                        "old": 'Zabbix Template "{0}" did not exist.'.format(name),
                        "new": 'Zabbix Template "{0}" created according definition.'.format(
                            name
                        ),
                    }
                }
                self.assertDictEqual(zabbix_template.present(name, {}), ret)

    @patch("salt.states.zabbix_template.CHANGE_STACK", [])
    def test_present_exists(self):
        """
        Test to ensure that named template is present and not changed
        """
        name = "A Testing Template"
        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        def side_effect_run_query(*args):
            """
            Differentiate between __salt__ exec module function calls with different parameters.
            """
            if args[0] == "template.get":
                return EXISTING_OBJ
            elif args[0] == "application.get":
                return ["non-empty"]

        with patch.dict(zabbix_template.__opts__, {"test": False}):
            with patch.dict(
                zabbix_template.__salt__,
                {
                    "zabbix.get_zabbix_id_mapper": MagicMock(
                        return_value={"template": "templateid"}
                    ),
                    "zabbix.substitute_params": MagicMock(
                        side_effect=SUBSTITUTE_PARAMS_EXISTS
                    ),
                    "zabbix.run_query": MagicMock(side_effect=side_effect_run_query),
                    "zabbix.compare_params": MagicMock(
                        return_value={"new": {}, "old": {}}
                    ),
                },
            ):
                ret["result"] = True
                ret[
                    "comment"
                ] = 'Zabbix Template "{0}" already exists and corresponds to a definition.'.format(
                    name
                )
                self.assertDictEqual(zabbix_template.present(name, {}), ret)

    @patch("salt.states.zabbix_template.CHANGE_STACK", [])
    def test_present_update(self):
        """
        Test to ensure that named template is present but must be updated
        """
        name = "A Testing Template"
        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        def side_effect_run_query(*args):
            """
            Differentiate between __salt__ exec module function calls with different parameters.
            """
            if args[0] == "template.get":
                return ["length of result is 1 = template exists"]
            elif args[0] == "template.update":
                return DIFF_PARAMS

        with patch.dict(zabbix_template.__opts__, {"test": False}):
            with patch.dict(
                zabbix_template.__salt__,
                {
                    "zabbix.get_zabbix_id_mapper": MagicMock(
                        return_value={"template": "templateid"}
                    ),
                    "zabbix.substitute_params": MagicMock(
                        side_effect=SUBSTITUTE_PARAMS_UPDATE
                    ),
                    "zabbix.run_query": MagicMock(side_effect=side_effect_run_query),
                    "zabbix.compare_params": MagicMock(return_value=DIFF_PARAMS),
                },
            ):
                ret["result"] = True
                ret["comment"] = 'Zabbix Template "{0}" updated.'.format(name)
                ret["changes"] = {
                    name: {
                        "old": 'Zabbix Template "{0}" differed.'.format(name),
                        "new": 'Zabbix Template "{0}" updated according definition.'.format(
                            name
                        ),
                    }
                }
                self.assertDictEqual(zabbix_template.present(name, {}), ret)

    def test_absent_test_mode(self):
        """
        Test to ensure that named template is absent in test mode
        """
        name = "A Testing Template"
        ret = {"name": name, "result": False, "comment": "", "changes": {}}
        with patch.dict(zabbix_template.__opts__, {"test": True}):
            with patch.dict(
                zabbix_template.__salt__,
                {"zabbix.get_object_id_by_params": MagicMock(return_value=11)},
            ):
                ret["result"] = True
                ret["comment"] = 'Zabbix Template "{0}" would be deleted.'.format(name)
                ret["changes"] = {
                    name: {
                        "old": 'Zabbix Template "{0}" exists.'.format(name),
                        "new": 'Zabbix Template "{0}" would be deleted.'.format(name),
                    }
                }
                self.assertDictEqual(zabbix_template.absent(name), ret)

    def test_absent(self):
        """
        Test to ensure that named template is absent
        """
        name = "A Testing Template"
        ret = {"name": name, "result": False, "comment": "", "changes": {}}
        with patch.dict(zabbix_template.__opts__, {"test": False}):
            with patch.dict(
                zabbix_template.__salt__,
                {"zabbix.get_object_id_by_params": MagicMock(return_value=False)},
            ):
                ret["result"] = True
                ret["comment"] = 'Zabbix Template "{0}" does not exist.'.format(name)
                self.assertDictEqual(zabbix_template.absent(name), ret)

            with patch.dict(
                zabbix_template.__salt__,
                {"zabbix.get_object_id_by_params": MagicMock(return_value=11)},
            ):
                with patch.dict(
                    zabbix_template.__salt__,
                    {"zabbix.run_query": MagicMock(return_value=True)},
                ):
                    ret["result"] = True
                    ret["comment"] = 'Zabbix Template "{0}" deleted.'.format(name)
                    ret["changes"] = {
                        name: {
                            "old": 'Zabbix Template "{0}" existed.'.format(name),
                            "new": 'Zabbix Template "{0}" deleted.'.format(name),
                        }
                    }
                    self.assertDictEqual(zabbix_template.absent(name), ret)



# -*- coding: utf-8 -*-

from __future__ import absolute_import

import pytest
import salt.utils.platform
from tests.support.case import ModuleCase
from tests.support.helpers import destructiveTest, slowTest
from tests.support.unit import skipIf


@skipIf(not salt.utils.platform.is_windows(), "windows test only")
@pytest.mark.windows_whitelisted
class WinDNSTest(ModuleCase):
    """
    Test for salt.modules.win_dns_client
    """

    @destructiveTest
    @slowTest
    def test_add_remove_dns(self):
        """
        Test add and removing a dns server
        """
        # Get a list of interfaces on the system
        interfaces = self.run_function("network.interfaces_names")
        skipIf(interfaces.count == 0, "This test requires a network interface")

        interface = interfaces[0]
        dns = "8.8.8.8"
        # add dns server
        self.assertTrue(
            self.run_function("win_dns_client.add_dns", [dns, interface], index=42)
        )

        srvs = self.run_function("win_dns_client.get_dns_servers", interface=interface)
        self.assertIn(dns, srvs)

        # remove dns server
        self.assertTrue(
            self.run_function("win_dns_client.rm_dns", [dns], interface=interface)
        )

        srvs = self.run_function("win_dns_client.get_dns_servers", interface=interface)
        self.assertNotIn(dns, srvs)



# -*- coding: utf-8 -*-
"""
integration tests for mac_service
"""

from __future__ import absolute_import, print_function, unicode_literals

from tests.support.case import ModuleCase
from tests.support.helpers import (
    destructiveTest,
    runs_on,
    skip_if_binaries_missing,
    skip_if_not_root,
    slowTest,
)


@skip_if_not_root
@runs_on(kernel="Darwin")
@skip_if_binaries_missing("launchctl", "plutil")
class MacServiceModuleTest(ModuleCase):
    """
    Validate the mac_service module
    """

    SERVICE_NAME = "com.apple.apsd"
    SERVICE_ENABLED = False

    def setUp(self):
        """
        Get current state of the test service
        """
        self.SERVICE_ENABLED = self.run_function("service.enabled", [self.SERVICE_NAME])

    def tearDown(self):
        """
        Reset the test service to the original state
        """
        if self.SERVICE_ENABLED:
            self.run_function("service.start", [self.SERVICE_NAME])
            self.run_function("service.enable", [self.SERVICE_NAME])
        else:
            self.run_function("service.stop", [self.SERVICE_NAME])
            self.run_function("service.disable", [self.SERVICE_NAME])

    @slowTest
    def test_show(self):
        """
        Test service.show
        """
        # Existing Service
        service_info = self.run_function("service.show", [self.SERVICE_NAME])
        self.assertIsInstance(service_info, dict)
        self.assertEqual(service_info["plist"]["Label"], self.SERVICE_NAME)

        # Missing Service
        self.assertIn(
            "Service not found", self.run_function("service.show", ["spongebob"])
        )

    @slowTest
    def test_launchctl(self):
        """
        Test service.launchctl
        """
        # Expected Functionality
        self.assertTrue(
            self.run_function("service.launchctl", ["error", "bootstrap", 64])
        )
        self.assertEqual(
            self.run_function(
                "service.launchctl", ["error", "bootstrap", 64], return_stdout=True
            ),
            "64: unknown error code",
        )

        # Raise an error
        self.assertIn(
            "Failed to error service",
            self.run_function("service.launchctl", ["error", "bootstrap"]),
        )

    @slowTest
    def test_list(self):
        """
        Test service.list
        """
        # Expected Functionality
        self.assertIn("PID", self.run_function("service.list"))
        self.assertIn(
            "{", self.run_function("service.list", ["com.apple.coreservicesd"])
        )

        # Service not found
        self.assertIn(
            "Service not found", self.run_function("service.list", ["spongebob"])
        )

    @destructiveTest
    @slowTest
    def test_enable(self):
        """
        Test service.enable
        """
        self.assertTrue(self.run_function("service.enable", [self.SERVICE_NAME]))

        self.assertIn(
            "Service not found", self.run_function("service.enable", ["spongebob"])
        )

    @destructiveTest
    @slowTest
    def test_disable(self):
        """
        Test service.disable
        """
        self.assertTrue(self.run_function("service.disable", [self.SERVICE_NAME]))

        self.assertIn(
            "Service not found", self.run_function("service.disable", ["spongebob"])
        )

    @destructiveTest
    @slowTest
    def test_start(self):
        """
        Test service.start
        Test service.stop
        Test service.status
        """
        self.assertTrue(self.run_function("service.start", [self.SERVICE_NAME]))

        self.assertIn(
            "Service not found", self.run_function("service.start", ["spongebob"])
        )

    @destructiveTest
    @slowTest
    def test_stop(self):
        """
        Test service.stop
        """
        self.assertTrue(self.run_function("service.stop", [self.SERVICE_NAME]))

        self.assertIn(
            "Service not found", self.run_function("service.stop", ["spongebob"])
        )

    @destructiveTest
    @slowTest
    def test_status(self):
        """
        Test service.status
        """
        # A running service
        self.assertTrue(self.run_function("service.start", [self.SERVICE_NAME]))
        self.assertTrue(
            self.run_function("service.status", [self.SERVICE_NAME]).isdigit()
        )

        # A stopped service
        self.assertTrue(self.run_function("service.stop", [self.SERVICE_NAME]))
        self.assertEqual("", self.run_function("service.status", [self.SERVICE_NAME]))

        # Service not found
        self.assertEqual("", self.run_function("service.status", ["spongebob"]))

    @slowTest
    def test_available(self):
        """
        Test service.available
        """
        self.assertTrue(self.run_function("service.available", [self.SERVICE_NAME]))
        self.assertFalse(self.run_function("service.available", ["spongebob"]))

    @slowTest
    def test_missing(self):
        """
        Test service.missing
        """
        self.assertFalse(self.run_function("service.missing", [self.SERVICE_NAME]))
        self.assertTrue(self.run_function("service.missing", ["spongebob"]))

    @destructiveTest
    @slowTest
    def test_enabled(self):
        """
        Test service.enabled
        """
        self.assertTrue(self.run_function("service.start", [self.SERVICE_NAME]))
        self.assertTrue(self.run_function("service.enabled", [self.SERVICE_NAME]))

        self.assertTrue(self.run_function("service.stop", [self.SERVICE_NAME]))
        self.assertFalse(self.run_function("service.enabled", [self.SERVICE_NAME]))

        self.assertFalse(self.run_function("service.enabled", ["spongebob"]))

    @destructiveTest
    @slowTest
    def test_disabled(self):
        """
        Test service.disabled
        """
        SERVICE_NAME = "com.apple.nfsd"
        self.assertTrue(self.run_function("service.start", [SERVICE_NAME]))
        self.assertFalse(self.run_function("service.disabled", [SERVICE_NAME]))

        self.assertTrue(self.run_function("service.disable", [SERVICE_NAME]))
        self.assertTrue(self.run_function("service.disabled", [SERVICE_NAME]))
        self.assertTrue(self.run_function("service.enable", [SERVICE_NAME]))

        self.assertFalse(self.run_function("service.disabled", ["spongebob"]))

    @slowTest
    def test_get_all(self):
        """
        Test service.get_all
        """
        services = self.run_function("service.get_all")
        self.assertIsInstance(services, list)
        self.assertIn(self.SERVICE_NAME, services)

    @slowTest
    def test_get_enabled(self):
        """
        Test service.get_enabled
        """
        services = self.run_function("service.get_enabled")
        self.assertIsInstance(services, list)
        self.assertIn("com.apple.coreservicesd", services)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Nicole Thomas <nicole@saltstack.com>
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.aliases as aliases
from salt.exceptions import SaltInvocationError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class AliasesTestCase(TestCase, LoaderModuleMockMixin):
    """
    TestCase for salt.modules.aliases module
    """

    mock_alias = [("foo", "bar@example.com", "")]
    mock_alias_mult = [
        ("foo", "bar@example.com", ""),
        ("hello", "world@earth.com, earth@world.com", ""),
    ]

    def setup_loader_modules(self):
        return {aliases: {}}

    def test_list_aliases(self):
        """
        Tests the return of a file containing one alias
        """
        with patch(
            "salt.modules.aliases.__parse_aliases",
            MagicMock(return_value=self.mock_alias),
        ):
            ret = {"foo": "bar@example.com"}
            self.assertEqual(aliases.list_aliases(), ret)

    def test_list_aliases_mult(self):
        """
        Tests the return of a file containing multiple aliases
        """
        with patch(
            "salt.modules.aliases.__parse_aliases",
            MagicMock(return_value=self.mock_alias_mult),
        ):
            ret = {
                "foo": "bar@example.com",
                "hello": "world@earth.com, earth@world.com",
            }
            self.assertEqual(aliases.list_aliases(), ret)

    def test_get_target(self):
        """
        Tests the target returned by an alias with one target
        """
        with patch(
            "salt.modules.aliases.__parse_aliases",
            MagicMock(return_value=self.mock_alias),
        ):
            ret = "bar@example.com"
            self.assertEqual(aliases.get_target("foo"), ret)

    def test_get_target_mult(self):
        """
        Tests the target returned by an alias with multiple targets
        """
        with patch(
            "salt.modules.aliases.__parse_aliases",
            MagicMock(return_value=self.mock_alias_mult),
        ):
            ret = "world@earth.com, earth@world.com"
            self.assertEqual(aliases.get_target("hello"), ret)

    def test_get_target_no_alias(self):
        """
        Tests return of an alias doesn't exist
        """
        with patch(
            "salt.modules.aliases.__parse_aliases",
            MagicMock(return_value=self.mock_alias),
        ):
            self.assertEqual(aliases.get_target("pizza"), "")

    def test_has_target(self):
        """
        Tests simple return known alias and target
        """
        with patch(
            "salt.modules.aliases.__parse_aliases",
            MagicMock(return_value=self.mock_alias),
        ):
            ret = aliases.has_target("foo", "bar@example.com")
            self.assertTrue(ret)

    def test_has_target_no_alias(self):
        """
        Tests return of empty alias and known target
        """
        with patch(
            "salt.modules.aliases.__parse_aliases",
            MagicMock(return_value=self.mock_alias),
        ):
            ret = aliases.has_target("", "bar@example.com")
            self.assertFalse(ret)

    def test_has_target_no_target(self):
        """
        Tests return of known alias and empty target
        """
        self.assertRaises(SaltInvocationError, aliases.has_target, "foo", "")

    def test_has_target_mult(self):
        """
        Tests return of multiple targets to one alias
        """
        with patch(
            "salt.modules.aliases.__parse_aliases",
            MagicMock(return_value=self.mock_alias_mult),
        ):
            ret = aliases.has_target("hello", "world@earth.com, earth@world.com")
            self.assertTrue(ret)

    def test_has_target_mult_differs(self):
        """
        Tests return of multiple targets to one alias in opposite order
        """
        with patch(
            "salt.modules.aliases.__parse_aliases",
            MagicMock(return_value=self.mock_alias_mult),
        ):
            ret = aliases.has_target("hello", "earth@world.com, world@earth.com")
            self.assertFalse(ret)

    def test_has_target_list_mult(self):
        """
        Tests return of target as same list to know alias
        """
        with patch(
            "salt.modules.aliases.__parse_aliases",
            MagicMock(return_value=self.mock_alias_mult),
        ):
            ret = aliases.has_target("hello", ["world@earth.com", "earth@world.com"])
            self.assertTrue(ret)

    def test_has_target_list_mult_differs(self):
        """
        Tests return of target as differing list to known alias
        """
        with patch(
            "salt.modules.aliases.__parse_aliases",
            MagicMock(return_value=self.mock_alias_mult),
        ):
            ret = aliases.has_target("hello", ["world@earth.com", "mars@space.com"])
            self.assertFalse(ret)

    def test_set_target_equal(self):
        """
        Tests return when target is already present
        """
        with patch(
            "salt.modules.aliases.get_target", MagicMock(return_value="bar@example.com")
        ):
            alias = "foo"
            target = "bar@example.com"
            ret = aliases.set_target(alias, target)
            self.assertTrue(ret)

    def test_set_target_empty_alias(self):
        """
        Tests return of empty alias
        """
        self.assertRaises(SaltInvocationError, aliases.set_target, "", "foo")

    def test_set_target_empty_target(self):
        """
        Tests return of known alias and empty target
        """
        self.assertRaises(SaltInvocationError, aliases.set_target, "foo", "")

    def test_rm_alias_absent(self):
        """
        Tests return when alias is not present
        """
        with patch("salt.modules.aliases.get_target", MagicMock(return_value="")):
            ret = aliases.rm_alias("foo")
            self.assertTrue(ret)
